@@ -25,8 +25,20 @@
  * THE SOFTWARE.
  */
 
-#ifdef DEBUG
-extern void NORETURN __fatal_error(const char *msg);
-#else
-#define __fatal_error(...)     for ( ;; ) {__WFI();}
+#ifndef MPERROR_H_
+#define MPERROR_H_
+
+#ifndef BOOTLOADER
+extern const mp_obj_type_t pyb_heartbeat_type;
 #endif
+
+extern void NORETURN __fatal_error(const char *msg);
+
+void mperror_init0 (void);
+void mperror_bootloader_check_reset_cause (void);
+void mperror_deinit_sfe_pin (void);
+void mperror_signal_error (void);
+void mperror_heartbeat_switch_off (void);
+void mperror_heartbeat_signal (void);
+
+#endif // MPERROR_H_
