@@ -25,38 +25,33 @@
  * THE SOFTWARE.
  */
 
-#include <stdio.h>
-#include <string.h>
-#include <stdarg.h>
-#include <errno.h>
 #include <stdint.h>
+#include <stdio.h>
+#include <errno.h>
+#include <string.h>
 
-#include "py/mpstate.h"
-#include "mpconfig.h"
+#include "py/mpconfig.h"
 #include MICROPY_HAL_H
-#include "nlr.h"
-#include "misc.h"
-#include "qstr.h"
-#include "obj.h"
-#include "runtime.h"
-#include "stream.h"
+#include "py/obj.h"
+#include "py/runtime.h"
+#include "py/objlist.h"
+#include "py/stream.h"
 #include "inc/hw_types.h"
-#include "inc/hw_gpio.h"
 #include "inc/hw_ints.h"
 #include "inc/hw_memmap.h"
 #include "inc/hw_uart.h"
 #include "rom_map.h"
 #include "interrupt.h"
 #include "prcm.h"
-#include "gpio.h"
 #include "uart.h"
-#include "pin.h"
 #include "pybuart.h"
 #include "pybioctl.h"
-#include "pybstdio.h"
+#include "pybsleep.h"
+#include "mpcallback.h"
 #include "mpexception.h"
+#include "py/mpstate.h"
 #include "osi.h"
-
+#include "utils.h"
 
 /// \moduleref pyb
 /// \class UART - duplex serial communication bus
@@ -91,23 +86,30 @@
 ///
 ///     uart.any()          # returns True if any characters waiting
 
-
+/******************************************************************************
+ DEFINE CONSTANTS
+ ******************************************************************************/
+#define PYBUART_TX_WAIT_MS                  1
+#define PYBUART_TX_MAX_TIMEOUT_MS           5
 
 /******************************************************************************
  DECLARE PRIVATE FUNCTIONS
  ******************************************************************************/
+STATIC void uart_init (pyb_uart_obj_t *self);
+STATIC bool uart_rx_wait (pyb_uart_obj_t *self, uint32_t timeout);
 STATIC void UARTGenericIntHandler(uint32_t uart_id);
 STATIC void UART0IntHandler(void);
 STATIC void UART1IntHandler(void);
+STATIC void uart_callback_enable (mp_obj_t self_in);
+STATIC void uart_callback_disable (mp_obj_t self_in);
 STATIC mp_obj_t pyb_uart_deinit(mp_obj_t self_in);
 
 /******************************************************************************
  DEFINE PRIVATE TYPES
  ******************************************************************************/
-
 struct _pyb_uart_obj_t {
     mp_obj_base_t base;
-    pyb_uart_t uart_id;
+    pyb_uart_id_t uart_id;
     uint reg;
     uint baudrate;
     uint config;
@@ -118,112 +120,26 @@ struct _pyb_uart_obj_t {
     uint16_t read_buf_len;              // len in chars; buf can hold len-1 chars
     volatile uint16_t read_buf_head;    // indexes first empty slot
     uint16_t read_buf_tail;             // indexes first full slot (not full if equals head)
-    bool enabled;
+    byte peripheral;
 };
 
-#define PYBUART_TX_WAIT_MS              1
-#define PYBUART_TX_MAX_TIMEOUT_MS       5
+/******************************************************************************
+ DECLARE PRIVATE DATA
+ ******************************************************************************/
+STATIC pyb_uart_obj_t pyb_uart_obj[PYB_NUM_UARTS] = {{.reg = UARTA0_BASE, .baudrate = 0, .peripheral = PRCM_UARTA0},
+                                                     {.reg = UARTA1_BASE, .baudrate = 0, .peripheral = PRCM_UARTA1}};
+STATIC const mp_cb_methods_t uart_cb_methods;
 
 /******************************************************************************
  DEFINE PUBLIC FUNCTIONS
  ******************************************************************************/
 void uart_init0 (void) {
-    for (int i = 0; i < MP_ARRAY_SIZE(MP_STATE_PORT(pyb_uart_obj_all)); i++) {
-        MP_STATE_PORT(pyb_uart_obj_all)[i] = NULL;
-    }
-}
-
-// unregister all interrupt sources
-void uart_deinit(void) {
-    for (int i = 0; i < MP_ARRAY_SIZE(MP_STATE_PORT(pyb_uart_obj_all)); i++) {
-        pyb_uart_obj_t *self = MP_STATE_PORT(pyb_uart_obj_all)[i];
-        if (self != NULL) {
-            pyb_uart_deinit(self);
-        }
-    }
-}
-
-// assumes Init parameters have been set up correctly
-bool uart_init2(pyb_uart_obj_t *self) {
-    uint uartPerh;
-
-    switch (self->uart_id) {
-    case PYB_UART_1:
-        self->reg = UARTA0_BASE;
-        uartPerh = PRCM_UARTA0;
-        MAP_PinTypeUART(PIN_55, PIN_MODE_3);
-        MAP_PinTypeUART(PIN_57, PIN_MODE_3);
-        MAP_UARTIntRegister(UARTA0_BASE, UART0IntHandler);
-        MAP_IntPrioritySet(INT_UARTA0, INT_PRIORITY_LVL_7);
-        break;
-    case PYB_UART_2:
-        self->reg = UARTA1_BASE;
-        uartPerh = PRCM_UARTA1;
-        MAP_PinTypeUART(PIN_58, PIN_MODE_6);
-        MAP_PinTypeUART(PIN_59, PIN_MODE_6);
-        MAP_UARTIntRegister(UARTA1_BASE, UART1IntHandler);
-        MAP_IntPrioritySet(INT_UARTA1, INT_PRIORITY_LVL_7);
-        break;
-    default:
-        return false;
-    }
-
-    // Enable the peripheral clock
-    MAP_PRCMPeripheralClkEnable(uartPerh, PRCM_RUN_MODE_CLK | PRCM_SLP_MODE_CLK);
-
-    // Reset the uart
-    MAP_PRCMPeripheralReset(uartPerh);
-
-    // Initialize the UART
-    MAP_UARTConfigSetExpClk(self->reg, MAP_PRCMPeripheralClockGet(uartPerh),
-                            self->baudrate, self->config);
-
-    // Enbale the FIFO
-    MAP_UARTFIFOEnable(self->reg);
-
-    // Configure the FIFO interrupt levels
-    MAP_UARTFIFOLevelSet(self->reg, UART_FIFO_TX4_8, UART_FIFO_RX4_8);
-    
-    // Configure the flow control mode
-    UARTFlowControlSet(self->reg, self->flowcontrol);
-
-    // Enable the RX and RX timeout interrupts
-    MAP_UARTIntEnable(self->reg, UART_INT_RX | UART_INT_RT);
-
-    self->enabled = true;
-
-    return true;
-}
-
-bool uart_init(pyb_uart_obj_t *self, uint baudrate) {
-    self->baudrate = baudrate;
-    self->config = UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE | UART_CONFIG_PAR_NONE;
-    self->flowcontrol = UART_FLOWCONTROL_NONE;
-    return uart_init2(self);
 }
 
 bool uart_rx_any(pyb_uart_obj_t *self) {
     return (self->read_buf_tail != self->read_buf_head || MAP_UARTCharsAvail(self->reg));
 }
 
-// Waits at most timeout milliseconds for at least 1 char to become ready for
-// reading (from buf or for direct reading).
-// Returns true if something available, false if not.
-STATIC bool uart_rx_wait(pyb_uart_obj_t *self, uint32_t timeout) {
-    for (;;) {
-        if (uart_rx_any(self)) {
-            return true; // have at least 1 char ready for reading
-        }
-        if (timeout > 0) {
-            HAL_Delay (1);
-            timeout--;
-        }
-        else {
-            return false;
-        }
-    }
-}
-
 int uart_rx_char(pyb_uart_obj_t *self) {
     if (self->read_buf_tail != self->read_buf_head) {
         // buffering via IRQ
@@ -266,29 +182,99 @@ void uart_tx_strn_cooked(pyb_uart_obj_t *self, const char *str, uint len) {
     }
 }
 
+mp_obj_t uart_callback_new (pyb_uart_obj_t *self, mp_obj_t handler, uint rxbuffer_size, mp_int_t priority) {
+    // disable the uart interrupts before updating anything
+    uart_callback_disable (self);
+
+    if (self->uart_id == PYB_UART_0) {
+        MAP_IntPrioritySet(INT_UARTA0, priority);
+        MAP_UARTIntRegister(self->reg, UART0IntHandler);
+    }
+    else {
+        MAP_IntPrioritySet(INT_UARTA1, priority);
+        MAP_UARTIntRegister(self->reg, UART1IntHandler);
+    }
+
+    // check the rx buffer size
+    if (rxbuffer_size > 0) {
+        // allocate the read buffer
+        self->read_buf_len = rxbuffer_size;
+        self->read_buf = m_new(byte, rxbuffer_size);
+    }
+
+    // create the callback
+    mp_obj_t _callback = mpcallback_new ((mp_obj_t)self, handler, &uart_cb_methods);
+
+    // enable the interrupts now
+    uart_callback_enable (self);
+
+    return _callback;
+}
+
+void uart_disable_all (void) {
+    for (int i = 0; i < PYB_NUM_UARTS; i++) {
+        // in case it's not clocked
+        MAP_PRCMPeripheralClkEnable(pyb_uart_obj[i].peripheral, PRCM_RUN_MODE_CLK | PRCM_SLP_MODE_CLK);
+        pyb_uart_deinit(&pyb_uart_obj[i]);
+    }
+}
+
 /******************************************************************************
  DEFINE PRIVATE FUNCTIONS
  ******************************************************************************/
+// assumes init parameters have been set up correctly
+STATIC void uart_init (pyb_uart_obj_t *self) {
+    // Enable the peripheral clock
+    MAP_PRCMPeripheralClkEnable(self->peripheral, PRCM_RUN_MODE_CLK | PRCM_SLP_MODE_CLK);
+
+    // Reset the uart
+    MAP_PRCMPeripheralReset(self->peripheral);
+
+    // Initialize the UART
+    MAP_UARTConfigSetExpClk(self->reg, MAP_PRCMPeripheralClockGet(self->peripheral),
+                            self->baudrate, self->config);
+
+    // Enable the FIFO
+    MAP_UARTFIFOEnable(self->reg);
+
+    // Configure the FIFO interrupt levels
+    MAP_UARTFIFOLevelSet(self->reg, UART_FIFO_TX4_8, UART_FIFO_RX4_8);
+
+    // Configure the flow control mode
+    UARTFlowControlSet(self->reg, self->flowcontrol);
+}
+
+// Waits at most timeout milliseconds for at least 1 char to become ready for
+// reading (from buf or for direct reading).
+// Returns true if something available, false if not.
+STATIC bool uart_rx_wait (pyb_uart_obj_t *self, uint32_t timeout) {
+    for ( ; ; ) {
+        if (uart_rx_any(self)) {
+            return true; // have at least 1 char ready for reading
+        }
+        if (timeout > 0) {
+            HAL_Delay (1);
+            timeout--;
+        }
+        else {
+            return false;
+        }
+    }
+}
 
 STATIC void UARTGenericIntHandler(uint32_t uart_id) {
-    pyb_uart_obj_t *self = MP_STATE_PORT(pyb_uart_obj_all)[uart_id];
+    pyb_uart_obj_t *self;
     uint32_t status;
 
-    if (self == NULL) {
-        // UART object has not been set, so we can't do anything, not
-        // even disable the IRQ.  This should never happen.
-        return;
-    }
-
+    self = &pyb_uart_obj[uart_id];
     status = MAP_UARTIntStatus(self->reg, true);
-
-    // Receive interrupt
+    // receive interrupt
     if (status & (UART_INT_RX | UART_INT_RT)) {
         MAP_UARTIntClear(self->reg, UART_INT_RX | UART_INT_RT);
         while (UARTCharsAvail(self->reg)) {
             int data = MAP_UARTCharGetNonBlocking(self->reg);
-            if (MICROPY_STDIO_UART == self->uart_id && data == user_interrupt_char) {
-                // raise exception when interrupts are finished
+            if (pyb_stdio_uart == self && data == user_interrupt_char) {
+                // raise an exception when interrupts are finished
                 mpexception_keyboard_nlr_jump();
             }
             else if (self->read_buf_len != 0) {
@@ -300,6 +286,9 @@ STATIC void UARTGenericIntHandler(uint32_t uart_id) {
                 }
             }
         }
+        // call the user defined handler
+        mp_obj_t _callback = mpcallback_find(self);
+        mpcallback_handler(_callback);
     }
 }
 
@@ -311,55 +300,66 @@ STATIC void UART1IntHandler(void) {
     UARTGenericIntHandler(1);
 }
 
+STATIC void uart_callback_enable (mp_obj_t self_in) {
+    pyb_uart_obj_t *self = self_in;
+    MAP_UARTIntClear(self->reg, UART_INT_RX | UART_INT_RT);
+    MAP_UARTIntEnable(self->reg, UART_INT_RX | UART_INT_RT);
+}
+
+STATIC void uart_callback_disable (mp_obj_t self_in) {
+    pyb_uart_obj_t *self = self_in;
+    MAP_UARTIntDisable(self->reg, UART_INT_RX | UART_INT_RT);
+}
+
 /******************************************************************************/
 /* Micro Python bindings                                                      */
 
-STATIC void pyb_uart_print(void (*print)(void *env, const char *fmt, ...), void *env, mp_obj_t self_in, mp_print_kind_t kind) {
+STATIC void pyb_uart_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
     pyb_uart_obj_t *self = self_in;
-    if (!self->enabled) {
-        print(env, "UART(%u)", self->uart_id);
-    } else {
-        print(env, "UART(%u, baudrate=%u, bits=", self->uart_id, self->baudrate);
+    if (self->baudrate > 0) {
+        mp_printf(print, "<UART%u, baudrate=%u, bits=", (self->uart_id + 1), self->baudrate);
         switch (self->config & UART_CONFIG_WLEN_MASK) {
         case UART_CONFIG_WLEN_5:
-            print(env, "5");
+            mp_print_str(print, "5");
             break;
         case UART_CONFIG_WLEN_6:
-            print(env, "6");
+            mp_print_str(print, "6");
             break;
         case UART_CONFIG_WLEN_7:
-            print(env, "7");
+            mp_print_str(print, "7");
             break;
         case UART_CONFIG_WLEN_8:
-            print(env, "8");
+            mp_print_str(print, "8");
             break;
         default:
             break;
         }
         if ((self->config & UART_CONFIG_PAR_MASK) == UART_CONFIG_PAR_NONE) {
-            print(env, ", parity=None");
+            mp_print_str(print, ", parity=None");
         } else {
-            print(env, ", parity=%u", (self->config & UART_CONFIG_PAR_MASK) == UART_CONFIG_PAR_EVEN ? 0 : 1);
+            mp_printf(print, ", parity=%u", (self->config & UART_CONFIG_PAR_MASK) == UART_CONFIG_PAR_EVEN ? 0 : 1);
         }
-        print(env, ", stop=%u, timeout=%u, timeout_char=%u, read_buf_len=%u)",
+        mp_printf(print, ", stop=%u, timeout=%u, timeout_char=%u, read_buf_len=%u>",
               (self->config & UART_CONFIG_STOP_MASK) == UART_CONFIG_STOP_ONE ? 1 : 2,
                self->timeout, self->timeout_char, self->read_buf_len);
     }
+    else {
+        mp_printf(print, "<UART%u>", (self->uart_id + 1));
+    }
 }
 
-/// \method init(baudrate, bits=8, parity=None, stop=1, *, timeout=1000, timeout_char=0, read_buf_len=64)
+/// \method init(baudrate, bits=8, parity=None, stop=1, *, timeout=1000, timeout_char=0)
 ///
 /// Initialise the UART bus with the given parameters:
 ///
 ///   - `baudrate` is the clock rate.
-///   - `bits` is the number of bits per byte, 7, 8 or 9.
+///   - `bits` is the number of bits per byte, 5, 6, 7, 8
 ///   - `parity` is the parity, `None`, 0 (even) or 1 (odd).
 ///   - `stop` is the number of stop bits, 1 or 2.
-///   - `flowcontrol` is the flow control mode, `None`, `UART.FLOW_TX`, 
-///     `UART.FLOW_RX', 'UART.FLOW_TXRX`.
-///   - `timeout` is the timeout in milliseconds to wait for the first character.
-///   - `timeout_char` is the timeout in milliseconds to wait between characters.
-///   - `read_buf_len` is the character length of the read buffer (0 to disable).
+///   - `flow` is the flow control mode, `None`, `UART.RTS`,
+///     `UART.CTS', or `UART.CTS | UART.RTS`
+///   - `timeout` is the timeout (in milliseconds) when waiting for the first character.
+///   - `timeout_char` is the timeout (in milliseconds) between characters.
 STATIC const mp_arg_t pyb_uart_init_args[] = {
     { MP_QSTR_baudrate,     MP_ARG_REQUIRED | MP_ARG_INT,  },
     { MP_QSTR_bits,         MP_ARG_KW_ONLY  | MP_ARG_INT,  {.u_int = 8} },
@@ -368,19 +368,28 @@ STATIC const mp_arg_t pyb_uart_init_args[] = {
     { MP_QSTR_flow,         MP_ARG_KW_ONLY  | MP_ARG_OBJ,  {.u_int = UART_FLOWCONTROL_NONE} },
     { MP_QSTR_timeout,      MP_ARG_KW_ONLY  | MP_ARG_INT,  {.u_int = 1000} },
     { MP_QSTR_timeout_char, MP_ARG_KW_ONLY  | MP_ARG_INT,  {.u_int = 1} },
-    { MP_QSTR_read_buf_len, MP_ARG_KW_ONLY  | MP_ARG_INT,  {.u_int = 64} },
 };
 
 STATIC mp_obj_t pyb_uart_init_helper(pyb_uart_obj_t *self, mp_uint_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
-    bool success;
-    
     // parse args
     mp_arg_val_t args[MP_ARRAY_SIZE(pyb_uart_init_args)];
     mp_arg_parse_all(n_args, pos_args, kw_args, MP_ARRAY_SIZE(pyb_uart_init_args), pyb_uart_init_args, args);
 
+    // set timeouts
+    self->timeout = args[5].u_int;
+    self->timeout_char = args[6].u_int;
+
+    // no read buffer for the moment
+    self->read_buf_head = 0;
+    self->read_buf_tail = 0;
+    self->read_buf_len = 0;
+    self->read_buf = NULL;
+
+    // get the baudrate
+    self->baudrate = args[0].u_int;
+
     // set the UART configuration values
     if (n_args > 1) {
-        self->baudrate = args[0].u_int;
         switch (args[1].u_int) {
         case 5:
             self->config = UART_CONFIG_WLEN_5;
@@ -395,7 +404,7 @@ STATIC mp_obj_t pyb_uart_init_helper(pyb_uart_obj_t *self, mp_uint_t n_args, con
             self->config = UART_CONFIG_WLEN_8;
             break;
         default:
-            nlr_raise(mp_obj_new_exception_msg(&mp_type_ValueError, mpexception_value_invalid_arguments));
+            goto error;
             break;
         }
         // Parity
@@ -406,45 +415,32 @@ STATIC mp_obj_t pyb_uart_init_helper(pyb_uart_obj_t *self, mp_uint_t n_args, con
         }
         // Stop bits
         self->config |= (args[3].u_int == 1 ? UART_CONFIG_STOP_ONE : UART_CONFIG_STOP_TWO);
-        
         // Flow control
+        if (args[4].u_int != UART_FLOWCONTROL_NONE || args[4].u_int != UART_FLOWCONTROL_TX ||
+            args[4].u_int != UART_FLOWCONTROL_RX || args[4].u_int != (UART_FLOWCONTROL_TX | UART_FLOWCONTROL_RX)) {
+            goto error;
+        }
         self->flowcontrol = args[4].u_int;
-        success = uart_init2(self);        
-    } else {
-        success = uart_init(self, args[0].u_int);
     }
-    
-    // init UART (if it fails, something weird happened)
-    if (!success) {
-        nlr_raise(mp_obj_new_exception_msg(&mp_type_OSError, mpexception_os_operation_failed));
+    else {
+        self->config = UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE | UART_CONFIG_PAR_NONE;
+        self->flowcontrol = UART_FLOWCONTROL_NONE;
     }
 
-    // set timeouts
-    self->timeout = args[5].u_int;
-    self->timeout_char = args[6].u_int;
+    // initialize and enable the uart
+    uart_init (self);
+    // register it with the sleep module
+    pybsleep_add ((const mp_obj_t)self, (WakeUpCB_t)uart_init);
 
-    // setup the read buffer
-    m_del(byte, self->read_buf, self->read_buf_len);
-    self->read_buf_head = 0;
-    self->read_buf_tail = 0;
-
-    if (args[7].u_int <= 0) {
-        // no read buffer
-        self->read_buf_len = 0;
-        self->read_buf = NULL;
-        MAP_UARTIntDisable(self->reg, UART_INT_RX | UART_INT_RT);
-    } else {
-        // read buffer using interrupts
-        self->read_buf_len = args[7].u_int;
-        self->read_buf = m_new(byte, args[7].u_int);
-    }
-   
     return mp_const_none;
+
+error:
+    nlr_raise(mp_obj_new_exception_msg(&mp_type_ValueError, mpexception_value_invalid_arguments));
 }
 
 /// \classmethod \constructor(bus, ...)
 ///
-/// Construct a UART object on the given bus.  `bus` can be 1-2
+/// Construct a UART object on the given bus id.  `bus id` can be 1 or 2
 /// With no additional parameters, the UART object is created but not
 /// initialised (it has the settings from the last initialisation of
 /// the bus, if any).
@@ -459,34 +455,23 @@ STATIC mp_obj_t pyb_uart_make_new(mp_obj_t type_in, mp_uint_t n_args, mp_uint_t
     // check arguments
     mp_arg_check_num(n_args, n_kw, 1, MP_ARRAY_SIZE(pyb_uart_init_args), true);
 
-    // work out port
-    pyb_uart_t uart_id = mp_obj_get_int(args[0]);
+    // work out the uart id
+    int32_t uart_id = mp_obj_get_int(args[0]) - 1;
 
-    if (uart_id < PYB_UART_1 || uart_id > MP_ARRAY_SIZE(MP_STATE_PORT(pyb_uart_obj_all))) {
+    if (uart_id < PYB_UART_0 || uart_id > PYB_UART_1) {
         nlr_raise(mp_obj_new_exception_msg(&mp_type_OSError, mpexception_os_resource_not_avaliable));
     }
 
-    // create object
-    pyb_uart_obj_t *self;
-    if (MP_STATE_PORT(pyb_uart_obj_all)[uart_id - 1] == NULL) {
-        // create new UART object
-        self = m_new0(pyb_uart_obj_t, 1);
-        self->base.type = &pyb_uart_type;
-        self->uart_id = uart_id;
-        self->enabled = false;
-        MP_STATE_PORT(pyb_uart_obj_all)[uart_id - 1] = self;
-    } else {
-        // reference existing UART object
-        self = MP_STATE_PORT(pyb_uart_obj_all)[uart_id - 1];
-    }
+    // get the correct uart instance
+    pyb_uart_obj_t *self = &pyb_uart_obj[uart_id];
+    self->base.type = &pyb_uart_type;
+    self->uart_id = uart_id;
 
     if (n_args > 1 || n_kw > 0) {
         // start the peripheral
         mp_map_t kw_args;
         mp_map_init_fixed_table(&kw_args, n_kw, args + n_args);
         pyb_uart_init_helper(self, n_args - 1, args + 1, &kw_args);
-    } else if (!self->enabled) {
-        nlr_raise(mp_obj_new_exception_msg(&mp_type_TypeError, mpexception_num_type_invalid_arguments));
     }
 
     return self;
@@ -501,29 +486,14 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_KW(pyb_uart_init_obj, 1, pyb_uart_init);
 /// Turn off the UART bus.
 STATIC mp_obj_t pyb_uart_deinit(mp_obj_t self_in) {
     pyb_uart_obj_t *self = self_in;
-    uint uartPerh;
 
-    switch (self->uart_id) {
-
-    case PYB_UART_1:
-        uartPerh = PRCM_UARTA0;
-        break;
-
-    case PYB_UART_2:
-        uartPerh = PRCM_UARTA1;
-        break;
-
-    default:
-        return mp_const_none;
-    }
-
-    self->enabled = false;
+    // unregister it with the sleep module
+    pybsleep_remove (self);
+    // invalidate the baudrate
+    self->baudrate = 0;
     MAP_UARTIntDisable(self->reg, UART_INT_RX | UART_INT_RT);
-    MAP_UARTIntClear(self->reg, UART_INT_RX | UART_INT_RT);
-    MAP_UARTIntUnregister(self->reg);
     MAP_UARTDisable(self->reg);
-    MAP_PRCMPeripheralClkDisable(uartPerh, PRCM_RUN_MODE_CLK | PRCM_SLP_MODE_CLK);
-
+    MAP_PRCMPeripheralClkDisable(self->peripheral, PRCM_RUN_MODE_CLK | PRCM_SLP_MODE_CLK);
     return mp_const_none;
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(pyb_uart_deinit_obj, pyb_uart_deinit);
@@ -540,6 +510,33 @@ STATIC mp_obj_t pyb_uart_any(mp_obj_t self_in) {
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(pyb_uart_any_obj, pyb_uart_any);
 
+/// \method callback(handler, value, priority)
+/// Creates a callback object associated with the uart
+/// min num of arguments is 1 (value). The value is the size of the rx buffer
+STATIC mp_obj_t pyb_uart_callback (mp_uint_t n_args, const mp_obj_t *pos_args, mp_map_t *kw_args) {
+    mp_arg_val_t args[mpcallback_INIT_NUM_ARGS];
+    mp_arg_parse_all(n_args - 1, pos_args + 1, kw_args, mpcallback_INIT_NUM_ARGS, mpcallback_init_args, args);
+
+    // check if any parameters were passed
+    pyb_uart_obj_t *self = pos_args[0];
+    mp_obj_t _callback = mpcallback_find((mp_obj_t)self);
+    if (kw_args->used > 0 || !_callback) {
+
+        // convert the priority to the correct value
+        uint priority = mpcallback_translate_priority (args[2].u_int);
+
+        // check the power mode
+        if (PYB_PWR_MODE_ACTIVE != args[4].u_int) {
+            nlr_raise(mp_obj_new_exception_msg(&mp_type_ValueError, mpexception_value_invalid_arguments));
+        }
+
+        // register a new callback
+        return uart_callback_new (self, args[1].u_obj, args[3].u_int, priority);
+    }
+    return _callback;
+}
+STATIC MP_DEFINE_CONST_FUN_OBJ_KW(pyb_uart_callback_obj, 1, pyb_uart_callback);
+
 /// \method writechar(char)
 /// Write a single character on the bus.  `char` is an integer to write.
 /// Return value: `None`.
@@ -551,7 +548,7 @@ STATIC mp_obj_t pyb_uart_writechar(mp_obj_t self_in, mp_obj_t char_in) {
 
     // send the character
     if (!uart_tx_char(self, data)) {
-        nlr_raise(mp_obj_new_exception_arg1(&mp_type_OSError, MP_OBJ_NEW_SMALL_INT(ETIMEDOUT)));
+        nlr_raise(mp_obj_new_exception_msg(&mp_type_OSError, mpexception_os_operation_failed));
     }
 
     return mp_const_none;
@@ -572,12 +569,24 @@ STATIC mp_obj_t pyb_uart_readchar(mp_obj_t self_in) {
 }
 STATIC MP_DEFINE_CONST_FUN_OBJ_1(pyb_uart_readchar_obj, pyb_uart_readchar);
 
+/// \method sendbreak()
+STATIC mp_obj_t pyb_uart_sendbreak(mp_obj_t self_in) {
+    pyb_uart_obj_t *self = self_in;
+    // send a break signal for at least 2 complete frames
+    MAP_UARTBreakCtl(self->reg, true);
+    UtilsDelay(UTILS_DELAY_US_TO_COUNT((22 * 1000000) / self->baudrate));
+    MAP_UARTBreakCtl(self->reg, false);
+    return mp_const_none;
+}
+STATIC MP_DEFINE_CONST_FUN_OBJ_1(pyb_uart_sendbreak_obj, pyb_uart_sendbreak);
+
+
 STATIC const mp_map_elem_t pyb_uart_locals_dict_table[] = {
     // instance methods
-
     { MP_OBJ_NEW_QSTR(MP_QSTR_init),        (mp_obj_t)&pyb_uart_init_obj },
     { MP_OBJ_NEW_QSTR(MP_QSTR_deinit),      (mp_obj_t)&pyb_uart_deinit_obj },
     { MP_OBJ_NEW_QSTR(MP_QSTR_any),         (mp_obj_t)&pyb_uart_any_obj },
+    { MP_OBJ_NEW_QSTR(MP_QSTR_callback),    (mp_obj_t)&pyb_uart_callback_obj },
 
     /// \method read([nbytes])
     { MP_OBJ_NEW_QSTR(MP_QSTR_read),        (mp_obj_t)&mp_stream_read_obj },
@@ -592,15 +601,11 @@ STATIC const mp_map_elem_t pyb_uart_locals_dict_table[] = {
 
     { MP_OBJ_NEW_QSTR(MP_QSTR_writechar),   (mp_obj_t)&pyb_uart_writechar_obj },
     { MP_OBJ_NEW_QSTR(MP_QSTR_readchar),    (mp_obj_t)&pyb_uart_readchar_obj },
+    { MP_OBJ_NEW_QSTR(MP_QSTR_sendbreak),   (mp_obj_t)&pyb_uart_sendbreak_obj },
 
     // class constants
-    { MP_OBJ_NEW_QSTR(MP_QSTR_UART1),       MP_OBJ_NEW_SMALL_INT(PYB_UART_1) },
-    { MP_OBJ_NEW_QSTR(MP_QSTR_UART2),       MP_OBJ_NEW_SMALL_INT(PYB_UART_2) },
-
-    { MP_OBJ_NEW_QSTR(MP_QSTR_FLOW_NONE),   MP_OBJ_NEW_SMALL_INT(UART_FLOWCONTROL_NONE) },
-    { MP_OBJ_NEW_QSTR(MP_QSTR_FLOW_TX),     MP_OBJ_NEW_SMALL_INT(UART_FLOWCONTROL_TX) },
-    { MP_OBJ_NEW_QSTR(MP_QSTR_FLOW_RX),     MP_OBJ_NEW_SMALL_INT(UART_FLOWCONTROL_RX) },
-    { MP_OBJ_NEW_QSTR(MP_QSTR_FLOW_TXRX),   MP_OBJ_NEW_SMALL_INT(UART_FLOWCONTROL_TX | UART_FLOWCONTROL_RX) },
+    { MP_OBJ_NEW_QSTR(MP_QSTR_CTS),         MP_OBJ_NEW_SMALL_INT(UART_FLOWCONTROL_TX) },
+    { MP_OBJ_NEW_QSTR(MP_QSTR_RTS),         MP_OBJ_NEW_SMALL_INT(UART_FLOWCONTROL_RX) },
 };
 
 STATIC MP_DEFINE_CONST_DICT(pyb_uart_locals_dict, pyb_uart_locals_dict_table);
@@ -623,7 +628,7 @@ STATIC mp_uint_t pyb_uart_read(mp_obj_t self_in, void *buf_in, mp_uint_t size, i
 
     // read the data
     byte *orig_buf = buf;
-    for (;;) {
+    for ( ; ; ) {
         *buf++ = uart_rx_char(self);
         if (--size == 0 || !uart_rx_wait(self, self->timeout_char)) {
             // return number of bytes read
@@ -638,7 +643,7 @@ STATIC mp_uint_t pyb_uart_write(mp_obj_t self_in, const void *buf_in, mp_uint_t
 
     // write the data
     if (!uart_tx_strn(self, buf, size)) {
-        nlr_raise(mp_obj_new_exception_arg1(&mp_type_OSError, MP_OBJ_NEW_SMALL_INT(ETIMEDOUT)));
+        nlr_raise(mp_obj_new_exception_msg(&mp_type_OSError, mpexception_os_operation_failed));
     }
     return size;
 }
@@ -669,6 +674,12 @@ STATIC const mp_stream_p_t uart_stream_p = {
     .is_text = false,
 };
 
+STATIC const mp_cb_methods_t uart_cb_methods = {
+    .init = pyb_uart_callback,
+    .enable = uart_callback_enable,
+    .disable = uart_callback_disable,
+};
+
 const mp_obj_type_t pyb_uart_type = {
     { &mp_type_type },
     .name = MP_QSTR_UART,
