@@ -66,6 +66,7 @@ extern "C" {
   
 #include <string.h>  
 #include "cc_pal.h"
+#include "debug.h"
 
 /*!
 	\def		MAX_CONCURRENT_ACTIONS
@@ -348,7 +349,11 @@ extern "C" {
     \note       belongs to \ref ported_sec
 
 */
+#ifdef DEBUG
 #define sl_DeviceEnablePreamble()		NwpPowerOnPreamble()
+#else
+#define sl_DeviceEnablePreamble()
+#endif
 
 /*!
     \brief		Enable the Network Processor
