@@ -6,7 +6,7 @@
  */
 
 // Receive single character
-int stdin_rx_chr(void) {
+int mp_hal_stdin_rx_chr(void) {
     unsigned char c = 0;
 #if MICROPY_MIN_USE_STDOUT
     int r = read(0, &c, 1);
@@ -16,7 +16,7 @@ int stdin_rx_chr(void) {
 }
 
 // Send string of given length
-void stdout_tx_strn(const char *str, mp_uint_t len) {
+void mp_hal_stdout_tx_strn(const char *str, mp_uint_t len) {
 #if MICROPY_MIN_USE_STDOUT
     int r = write(1, str, len);
     (void)r;
