@@ -32,6 +32,7 @@
 
 #include "py/binary.h"
 #include "py/smallint.h"
+#include "py/objint.h"
 
 // Helpers to work with binary-encoded data
 
@@ -39,8 +40,8 @@
 #define alignof(type) offsetof(struct { char c; type t; }, t)
 #endif
 
-int mp_binary_get_size(char struct_type, char val_type, mp_uint_t *palign) {
-    int size = 0;
+size_t mp_binary_get_size(char struct_type, char val_type, mp_uint_t *palign) {
+    size_t size = 0;
     int align = 1;
     switch (struct_type) {
         case '<': case '>':
@@ -141,6 +142,9 @@ mp_obj_t mp_binary_get_val_array(char typecode, void *p, mp_uint_t index) {
         case 'd':
             return mp_obj_new_float(((double*)p)[index]);
 #endif
+        // Extension to CPython: array of objects
+        case 'O':
+            return ((mp_obj_t*)p)[index];
     }
     return MP_OBJ_NEW_SMALL_INT(val);
 }
@@ -175,7 +179,7 @@ mp_obj_t mp_binary_get_val(char struct_type, char val_type, byte **ptr) {
     byte *p = *ptr;
     mp_uint_t align;
 
-    int size = mp_binary_get_size(struct_type, val_type, &align);
+    size_t size = mp_binary_get_size(struct_type, val_type, &align);
     if (struct_type == '@') {
         // Make pointer aligned
         p = (byte*)(((mp_uint_t)p + align - 1) & ~((mp_uint_t)align - 1));
@@ -240,7 +244,7 @@ void mp_binary_set_val(char struct_type, char val_type, mp_obj_t val_in, byte **
     byte *p = *ptr;
     mp_uint_t align;
 
-    int size = mp_binary_get_size(struct_type, val_type, &align);
+    size_t size = mp_binary_get_size(struct_type, val_type, &align);
     if (struct_type == '@') {
         // Make pointer aligned
         p = (byte*)(((mp_uint_t)p + align - 1) & ~((mp_uint_t)align - 1));
@@ -279,11 +283,18 @@ void mp_binary_set_val(char struct_type, char val_type, mp_obj_t val_in, byte **
         }
 #endif
         default:
-            // we handle large ints here by calling the truncated accessor
+            #if MICROPY_LONGINT_IMPL != MICROPY_LONGINT_IMPL_NONE
             if (MP_OBJ_IS_TYPE(val_in, &mp_type_int)) {
-                val = mp_obj_int_get_truncated(val_in);
-            } else {
+                mp_obj_int_to_bytes_impl(val_in, struct_type == '>', size, p);
+                return;
+            } else
+            #endif
+            {
                 val = mp_obj_get_int(val_in);
+                // sign extend if needed
+                if (BYTES_PER_WORD < 8 && size > sizeof(val) && is_signed(val_type) && (mp_int_t)val < 0) {
+                    memset(p + sizeof(val), 0xff, size - sizeof(val));
+                }
             }
     }
 
@@ -300,6 +311,10 @@ void mp_binary_set_val_array(char typecode, void *p, mp_uint_t index, mp_obj_t v
             ((double*)p)[index] = mp_obj_get_float(val_in);
             break;
 #endif
+        // Extension to CPython: array of objects
+        case 'O':
+            ((mp_obj_t*)p)[index] = val_in;
+            break;
         default:
             mp_binary_set_val_array_from_int(typecode, p, index, mp_obj_get_int(val_in));
     }
