@@ -43,7 +43,7 @@ STATIC mp_uint_t str_buf_next_byte(mp_lexer_str_buf_t *sb) {
 
 STATIC void str_buf_free(mp_lexer_str_buf_t *sb) {
     if (sb->free_len > 0) {
-        m_free((char*)sb->src_beg, sb->free_len);
+        m_del(char, (char*)sb->src_beg, sb->free_len);
     }
     m_del_obj(mp_lexer_str_buf_t, sb);
 }
