@@ -68,8 +68,6 @@ MATH_FUN_2(pow, pow)
 MATH_FUN_1(exp, exp)
 /// \function expm1(x)
 MATH_FUN_1(expm1, expm1)
-/// \function log(x)
-MATH_FUN_1(log, log)
 /// \function log2(x)
 MATH_FUN_1(log2, log2)
 /// \function log10(x)
@@ -120,6 +118,7 @@ MATH_FUN_1_TO_BOOL(isnan, isnan)
 MATH_FUN_1_TO_INT(trunc, trunc)
 /// \function ldexp(x, exp)
 MATH_FUN_2(ldexp, ldexp)
+#if MICROPY_PY_MATH_SPECIAL_FUNCTIONS
 /// \function erf(x)
 /// Return the error function of `x`.
 MATH_FUN_1(erf, erf)
@@ -132,8 +131,22 @@ MATH_FUN_1(gamma, tgamma)
 /// \function lgamma(x)
 /// return the natural logarithm of the gamma function of `x`.
 MATH_FUN_1(lgamma, lgamma)
+#endif
 //TODO: factorial, fsum
 
+// Function that takes a variable number of arguments
+
+// log(x[, base])
+STATIC mp_obj_t mp_math_log(mp_uint_t n_args, const mp_obj_t *args) {
+    mp_float_t l = MICROPY_FLOAT_C_FUN(log)(mp_obj_get_float(args[0]));
+    if (n_args == 1) {
+        return mp_obj_new_float(l);
+    } else {
+        return mp_obj_new_float(l / MICROPY_FLOAT_C_FUN(log)(mp_obj_get_float(args[1])));
+    }
+}
+STATIC MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(mp_math_log_obj, 1, 2, mp_math_log);
+
 // Functions that return a tuple
 
 /// \function frexp(x)
@@ -211,10 +224,12 @@ STATIC const mp_map_elem_t mp_module_math_globals_table[] = {
     { MP_OBJ_NEW_QSTR(MP_QSTR_trunc), (mp_obj_t)&mp_math_trunc_obj },
     { MP_OBJ_NEW_QSTR(MP_QSTR_radians), (mp_obj_t)&mp_math_radians_obj },
     { MP_OBJ_NEW_QSTR(MP_QSTR_degrees), (mp_obj_t)&mp_math_degrees_obj },
+    #if MICROPY_PY_MATH_SPECIAL_FUNCTIONS
     { MP_OBJ_NEW_QSTR(MP_QSTR_erf), (mp_obj_t)&mp_math_erf_obj },
     { MP_OBJ_NEW_QSTR(MP_QSTR_erfc), (mp_obj_t)&mp_math_erfc_obj },
     { MP_OBJ_NEW_QSTR(MP_QSTR_gamma), (mp_obj_t)&mp_math_gamma_obj },
     { MP_OBJ_NEW_QSTR(MP_QSTR_lgamma), (mp_obj_t)&mp_math_lgamma_obj },
+    #endif
 };
 
 STATIC MP_DEFINE_CONST_DICT(mp_module_math_globals, mp_module_math_globals_table);
