@@ -31,17 +31,22 @@
 #include "py/objtuple.h"
 #include "py/objstr.h"
 #include "py/objint.h"
-#include "py/pfenv.h"
 #include "py/stream.h"
 
 #if MICROPY_PY_SYS
 
+#include "genhdr/mpversion.h"
+
 /// \module sys - system specific functions
 
 // defined per port; type of these is irrelevant, just need pointer
-extern mp_uint_t mp_sys_stdin_obj;
-extern mp_uint_t mp_sys_stdout_obj;
-extern mp_uint_t mp_sys_stderr_obj;
+extern struct _mp_dummy_t mp_sys_stdin_obj;
+extern struct _mp_dummy_t mp_sys_stdout_obj;
+extern struct _mp_dummy_t mp_sys_stderr_obj;
+
+#if MICROPY_PY_IO
+const mp_print_t mp_sys_stdout_print = {&mp_sys_stdout_obj, (mp_print_strn_t)mp_stream_write};
+#endif
 
 /// \constant version - Python language version that this implementation conforms to, as a string
 STATIC const MP_DEFINE_STR_OBJ(version_obj, "3.4.0");
@@ -50,6 +55,34 @@ STATIC const MP_DEFINE_STR_OBJ(version_obj, "3.4.0");
 #define I(n) MP_OBJ_NEW_SMALL_INT(n)
 // TODO: CPython is now at 5-element array, but save 2 els so far...
 STATIC const mp_obj_tuple_t mp_sys_version_info_obj = {{&mp_type_tuple}, 3, {I(3), I(4), I(0)}};
+
+// sys.implementation object
+// this holds the MicroPython version
+STATIC const mp_obj_tuple_t mp_sys_implementation_version_info_obj = {
+    {&mp_type_tuple},
+    3,
+    { I(MICROPY_VERSION_MAJOR), I(MICROPY_VERSION_MINOR), I(MICROPY_VERSION_MICRO) }
+};
+#if MICROPY_PY_ATTRTUPLE
+STATIC const qstr impl_fields[] = { MP_QSTR_name, MP_QSTR_version };
+STATIC MP_DEFINE_ATTRTUPLE(
+    mp_sys_implementation_obj,
+    impl_fields,
+    2,
+        MP_OBJ_NEW_QSTR(MP_QSTR_micropython),
+        (mp_obj_t)&mp_sys_implementation_version_info_obj
+);
+#else
+STATIC const mp_obj_tuple_t mp_sys_implementation_obj = {
+    {&mp_type_tuple},
+    2,
+    {
+        MP_OBJ_NEW_QSTR(MP_QSTR_micropython),
+        (mp_obj_t)&mp_sys_implementation_version_info_obj,
+    }
+};
+#endif
+
 #undef I
 
 #ifdef MICROPY_PY_SYS_PLATFORM
@@ -78,18 +111,36 @@ STATIC mp_obj_t mp_sys_print_exception(mp_uint_t n_args, const mp_obj_t *args) {
         stream_obj = args[1];
     }
 
-    pfenv_t pfenv;
-    pfenv.data = stream_obj;
-    pfenv.print_strn = (void (*)(void *, const char *, mp_uint_t))mp_stream_write;
-    mp_obj_print_exception((void (*)(void *env, const char *fmt, ...))pfenv_printf, &pfenv, args[0]);
+    mp_print_t print = {stream_obj, (mp_print_strn_t)mp_stream_write};
+    mp_obj_print_exception(&print, args[0]);
     #else
-    mp_obj_print_exception(printf_wrapper, NULL, args[0]);
+    mp_obj_print_exception(&mp_plat_print, args[0]);
     #endif
 
     return mp_const_none;
 }
 MP_DEFINE_CONST_FUN_OBJ_VAR_BETWEEN(mp_sys_print_exception_obj, 1, 2, mp_sys_print_exception);
 
+#if MICROPY_PY_SYS_EXC_INFO
+STATIC mp_obj_t mp_sys_exc_info(void) {
+    mp_obj_t cur_exc = MP_STATE_VM(cur_exception);
+    mp_obj_tuple_t *t = mp_obj_new_tuple(3, NULL);
+
+    if (cur_exc == MP_OBJ_NULL) {
+        t->items[0] = mp_const_none;
+        t->items[1] = mp_const_none;
+        t->items[2] = mp_const_none;
+        return t;
+    }
+
+    t->items[0] = mp_obj_get_type(cur_exc);
+    t->items[1] = cur_exc;
+    t->items[2] = mp_const_none;
+    return t;
+}
+MP_DEFINE_CONST_FUN_OBJ_0(mp_sys_exc_info_obj, mp_sys_exc_info);
+#endif
+
 STATIC const mp_map_elem_t mp_module_sys_globals_table[] = {
     { MP_OBJ_NEW_QSTR(MP_QSTR___name__), MP_OBJ_NEW_QSTR(MP_QSTR_sys) },
 
@@ -97,6 +148,7 @@ STATIC const mp_map_elem_t mp_module_sys_globals_table[] = {
     { MP_OBJ_NEW_QSTR(MP_QSTR_argv), (mp_obj_t)&MP_STATE_VM(mp_sys_argv_obj) },
     { MP_OBJ_NEW_QSTR(MP_QSTR_version), (mp_obj_t)&version_obj },
     { MP_OBJ_NEW_QSTR(MP_QSTR_version_info), (mp_obj_t)&mp_sys_version_info_obj },
+    { MP_OBJ_NEW_QSTR(MP_QSTR_implementation), (mp_obj_t)&mp_sys_implementation_obj },
 #ifdef MICROPY_PY_SYS_PLATFORM
     { MP_OBJ_NEW_QSTR(MP_QSTR_platform), (mp_obj_t)&platform_obj },
 #endif
@@ -131,6 +183,10 @@ STATIC const mp_map_elem_t mp_module_sys_globals_table[] = {
     { MP_OBJ_NEW_QSTR(MP_QSTR_stderr), (mp_obj_t)&mp_sys_stderr_obj },
 #endif
 
+    #if MICROPY_PY_SYS_EXC_INFO
+    { MP_OBJ_NEW_QSTR(MP_QSTR_exc_info), (mp_obj_t)&mp_sys_exc_info_obj },
+    #endif
+
     /*
      * Extensions to CPython
      */
