@@ -97,7 +97,7 @@ STATIC mp_uint_t mpn_shl(mpz_dig_t *idig, mpz_dig_t *jdig, mp_uint_t jlen, mp_ui
 
     // work out length of result
     jlen += n_whole;
-    if (idig[jlen - 1] == 0) {
+    while (jlen != 0 && idig[jlen - 1] == 0) {
         jlen--;
     }
 
@@ -572,6 +572,9 @@ void mpz_deinit(mpz_t *z) {
     }
 }
 
+#if 0
+these functions are unused
+
 mpz_t *mpz_zero(void) {
     mpz_t *z = m_new_obj(mpz_t);
     mpz_init_zero(z);
@@ -603,8 +606,9 @@ mpz_t *mpz_from_str(const char *str, mp_uint_t len, bool neg, mp_uint_t base) {
     mpz_set_from_str(z, str, len, neg, base);
     return z;
 }
+#endif
 
-void mpz_free(mpz_t *z) {
+STATIC void mpz_free(mpz_t *z) {
     if (z != NULL) {
         m_del(mpz_dig_t, z->dig, z->alloc);
         m_del_obj(mpz_t, z);
@@ -627,7 +631,7 @@ STATIC void mpz_need_dig(mpz_t *z, mp_uint_t need) {
     }
 }
 
-mpz_t *mpz_clone(const mpz_t *src) {
+STATIC mpz_t *mpz_clone(const mpz_t *src) {
     mpz_t *z = m_new_obj(mpz_t);
     z->neg = src->neg;
     z->fixed_dig = 0;
@@ -803,6 +807,9 @@ bool mpz_is_zero(const mpz_t *z) {
     return z->len == 0;
 }
 
+#if 0
+these functions are unused
+
 bool mpz_is_pos(const mpz_t *z) {
     return z->len > 0 && z->neg == 0;
 }
@@ -818,6 +825,7 @@ bool mpz_is_odd(const mpz_t *z) {
 bool mpz_is_even(const mpz_t *z) {
     return z->len == 0 || (z->dig[0] & 1) == 0;
 }
+#endif
 
 int mpz_cmp(const mpz_t *z1, const mpz_t *z2) {
     // to catch comparison of -0 with +0
@@ -921,6 +929,17 @@ mpz_t *mpz_pow(const mpz_t *lhs, const mpz_t *rhs) {
     mpz_pow_inpl(z, lhs, rhs);
     return z;
 }
+
+/* computes new integers in quo and rem such that:
+       quo * rhs + rem = lhs
+       0 <= rem < rhs
+   can have lhs, rhs the same
+*/
+void mpz_divmod(const mpz_t *lhs, const mpz_t *rhs, mpz_t **quo, mpz_t **rem) {
+    *quo = mpz_zero();
+    *rem = mpz_zero();
+    mpz_divmod_inpl(*quo, *rem, lhs, rhs);
+}
 #endif
 
 /* computes dest = abs(z)
@@ -1155,8 +1174,7 @@ void mpz_xor_inpl(mpz_t *dest, const mpz_t *lhs, const mpz_t *rhs) {
 /* computes dest = lhs * rhs
    can have dest, lhs, rhs the same
 */
-void mpz_mul_inpl(mpz_t *dest, const mpz_t *lhs, const mpz_t *rhs)
-{
+void mpz_mul_inpl(mpz_t *dest, const mpz_t *lhs, const mpz_t *rhs) {
     if (lhs->len == 0 || rhs->len == 0) {
         mpz_set_from_int(dest, 0);
         return;
@@ -1205,7 +1223,7 @@ void mpz_pow_inpl(mpz_t *dest, const mpz_t *lhs, const mpz_t *rhs) {
     mpz_set_from_int(dest, 1);
 
     while (n->len > 0) {
-        if (mpz_is_odd(n)) {
+        if ((n->dig[0] & 1) != 0) {
             mpz_mul_inpl(dest, dest, x);
         }
         n->len = mpn_shr(n->dig, n->dig, n->len, 1);
@@ -1219,6 +1237,9 @@ void mpz_pow_inpl(mpz_t *dest, const mpz_t *lhs, const mpz_t *rhs) {
     mpz_free(n);
 }
 
+#if 0
+these functions are unused
+
 /* computes gcd(z1, z2)
    based on Knuth's modified gcd algorithm (I think?)
    gcd(z1, z2) >= 0
@@ -1294,17 +1315,7 @@ mpz_t *mpz_lcm(const mpz_t *z1, const mpz_t *z2) {
     rem->neg = 0;
     return rem;
 }
-
-/* computes new integers in quo and rem such that:
-       quo * rhs + rem = lhs
-       0 <= rem < rhs
-   can have lhs, rhs the same
-*/
-void mpz_divmod(const mpz_t *lhs, const mpz_t *rhs, mpz_t **quo, mpz_t **rem) {
-    *quo = mpz_zero();
-    *rem = mpz_zero();
-    mpz_divmod_inpl(*quo, *rem, lhs, rhs);
-}
+#endif
 
 /* computes new integers in quo and rem such that:
        quo * rhs + rem = lhs
@@ -1414,6 +1425,40 @@ bool mpz_as_uint_checked(const mpz_t *i, mp_uint_t *value) {
     return true;
 }
 
+// writes at most len bytes to buf (so buf should be zeroed before calling)
+void mpz_as_bytes(const mpz_t *z, bool big_endian, mp_uint_t len, byte *buf) {
+    byte *b = buf;
+    if (big_endian) {
+        b += len;
+    }
+    mpz_dig_t *zdig = z->dig;
+    int bits = 0;
+    mpz_dbl_dig_t d = 0;
+    mpz_dbl_dig_t carry = 1;
+    for (mp_uint_t zlen = z->len; zlen > 0; --zlen) {
+        bits += DIG_SIZE;
+        d = (d << DIG_SIZE) | *zdig++;
+        for (; bits >= 8; bits -= 8, d >>= 8) {
+            mpz_dig_t val = d;
+            if (z->neg) {
+                val = (~val & 0xff) + carry;
+                carry = val >> 8;
+            }
+            if (big_endian) {
+                *--b = val;
+                if (b == buf) {
+                    return;
+                }
+            } else {
+                *b++ = val;
+                if (b == buf + len) {
+                    return;
+                }
+            }
+        }
+    }
+}
+
 #if MICROPY_PY_BUILTINS_FLOAT
 mp_float_t mpz_as_float(const mpz_t *i) {
     mp_float_t val = 0;
