@@ -60,21 +60,21 @@ STATIC mp_obj_t closure_call(mp_obj_t self_in, mp_uint_t n_args, mp_uint_t n_kw,
 }
 
 #if MICROPY_ERROR_REPORTING == MICROPY_ERROR_REPORTING_DETAILED
-STATIC void closure_print(void (*print)(void *env, const char *fmt, ...), void *env, mp_obj_t o_in, mp_print_kind_t kind) {
+STATIC void closure_print(const mp_print_t *print, mp_obj_t o_in, mp_print_kind_t kind) {
     (void)kind;
     mp_obj_closure_t *o = o_in;
-    print(env, "<closure ");
-    mp_obj_print_helper(print, env, o->fun, PRINT_REPR);
-    print(env, " at %p, n_closed=%u ", o, o->n_closed);
+    mp_print_str(print, "<closure ");
+    mp_obj_print_helper(print, o->fun, PRINT_REPR);
+    mp_printf(print, " at %p, n_closed=%u ", o, o->n_closed);
     for (mp_uint_t i = 0; i < o->n_closed; i++) {
         if (o->closed[i] == MP_OBJ_NULL) {
-            print(env, "(nil)");
+            mp_print_str(print, "(nil)");
         } else {
-            mp_obj_print_helper(print, env, o->closed[i], PRINT_REPR);
+            mp_obj_print_helper(print, o->closed[i], PRINT_REPR);
         }
-        print(env, " ");
+        mp_print_str(print, " ");
     }
-    print(env, ">");
+    mp_print_str(print, ">");
 }
 #endif
 
