@@ -38,7 +38,7 @@
 #include "py/gc.h"
 
 // Instance of MemoryError exception - needed by mp_malloc_fail
-const mp_obj_exception_t mp_const_MemoryError_obj = {{&mp_type_MemoryError}, MP_OBJ_NULL, mp_const_empty_tuple};
+const mp_obj_exception_t mp_const_MemoryError_obj = {{&mp_type_MemoryError}, 0, 0, MP_OBJ_NULL, mp_const_empty_tuple};
 
 // Optionally allocated buffer for storing the first argument of an exception
 // allocated when the heap is locked.
@@ -64,7 +64,7 @@ mp_obj_t mp_alloc_emergency_exception_buf(mp_obj_t size_in) {
     mp_int_t size = mp_obj_get_int(size_in);
     void *buf = NULL;
     if (size > 0) {
-        buf = m_malloc(size);
+        buf = m_new(byte, size);
     }
 
     int old_size = mp_emergency_exception_buf_size;
@@ -78,7 +78,7 @@ mp_obj_t mp_alloc_emergency_exception_buf(mp_obj_t size_in) {
     MICROPY_END_ATOMIC_SECTION(atomic_state);
 
     if (old_buf != NULL) {
-        m_free(old_buf, old_size);
+        m_del(byte, old_buf, old_size);
     }
     return mp_const_none;
 }
@@ -88,30 +88,30 @@ mp_obj_t mp_alloc_emergency_exception_buf(mp_obj_t size_in) {
 // Instance of GeneratorExit exception - needed by generator.close()
 // This would belong to objgenerator.c, but to keep mp_obj_exception_t
 // definition module-private so far, have it here.
-const mp_obj_exception_t mp_const_GeneratorExit_obj = {{&mp_type_GeneratorExit}, MP_OBJ_NULL, mp_const_empty_tuple};
+const mp_obj_exception_t mp_const_GeneratorExit_obj = {{&mp_type_GeneratorExit}, 0, 0, MP_OBJ_NULL, mp_const_empty_tuple};
 
-STATIC void mp_obj_exception_print(void (*print)(void *env, const char *fmt, ...), void *env, mp_obj_t o_in, mp_print_kind_t kind) {
+STATIC void mp_obj_exception_print(const mp_print_t *print, mp_obj_t o_in, mp_print_kind_t kind) {
     mp_obj_exception_t *o = o_in;
     mp_print_kind_t k = kind & ~PRINT_EXC_SUBCLASS;
     bool is_subclass = kind & PRINT_EXC_SUBCLASS;
     if (!is_subclass && (k == PRINT_REPR || k == PRINT_EXC)) {
-        print(env, "%s", qstr_str(o->base.type->name));
+        mp_print_str(print, qstr_str(o->base.type->name));
     }
 
     if (k == PRINT_EXC) {
-        print(env, ": ");
+        mp_print_str(print, ": ");
     }
 
     if (k == PRINT_STR || k == PRINT_EXC) {
         if (o->args == NULL || o->args->len == 0) {
-            print(env, "");
+            mp_print_str(print, "");
             return;
         } else if (o->args->len == 1) {
-            mp_obj_print_helper(print, env, o->args->items[0], PRINT_STR);
+            mp_obj_print_helper(print, o->args->items[0], PRINT_STR);
             return;
         }
     }
-    mp_obj_tuple_print(print, env, o->args, kind);
+    mp_obj_tuple_print(print, o->args, kind);
 }
 
 mp_obj_t mp_obj_exception_make_new(mp_obj_t type_in, mp_uint_t n_args, mp_uint_t n_kw, const mp_obj_t *args) {
@@ -126,7 +126,7 @@ mp_obj_t mp_obj_exception_make_new(mp_obj_t type_in, mp_uint_t n_args, mp_uint_t
         o->args = mp_obj_new_tuple(n_args, args);
     }
     o->base.type = type_in;
-    o->traceback = MP_OBJ_NULL;
+    o->traceback_data = NULL;
     return o;
 }
 
@@ -140,7 +140,11 @@ mp_obj_t mp_obj_exception_get_value(mp_obj_t self_in) {
     }
 }
 
-STATIC void exception_load_attr(mp_obj_t self_in, qstr attr, mp_obj_t *dest) {
+STATIC void exception_attr(mp_obj_t self_in, qstr attr, mp_obj_t *dest) {
+    if (dest[0] != MP_OBJ_NULL) {
+        // not load attribute
+        return;
+    }
     mp_obj_exception_t *self = self_in;
     if (attr == MP_QSTR_args) {
         dest[0] = self->args;
@@ -168,7 +172,7 @@ const mp_obj_type_t mp_type_BaseException = {
     .name = MP_QSTR_BaseException,
     .print = mp_obj_exception_print,
     .make_new = mp_obj_exception_make_new,
-    .load_attr = exception_load_attr,
+    .attr = exception_attr,
     .locals_dict = (mp_obj_t)&exc_locals_dict,
 };
 
@@ -181,7 +185,7 @@ const mp_obj_type_t mp_type_ ## exc_name = { \
     .name = MP_QSTR_ ## exc_name, \
     .print = mp_obj_exception_print, \
     .make_new = mp_obj_exception_make_new, \
-    .load_attr = exception_load_attr, \
+    .attr = exception_attr, \
     .bases_tuple = (mp_obj_t)&mp_type_ ## base_name ## _base_tuple, \
 };
 
@@ -248,8 +252,16 @@ MP_DEFINE_EXCEPTION(Exception, BaseException)
       */
   //MP_DEFINE_EXCEPTION(SystemError, Exception)
   MP_DEFINE_EXCEPTION(TypeError, Exception)
+#if MICROPY_EMIT_NATIVE
+    MP_DEFINE_EXCEPTION_BASE(TypeError)
+    MP_DEFINE_EXCEPTION(ViperTypeError, TypeError)
+#endif
   MP_DEFINE_EXCEPTION(ValueError, Exception)
-    //TODO: Implement UnicodeErrors which take arguments
+#if MICROPY_PY_BUILTINS_STR_UNICODE
+    MP_DEFINE_EXCEPTION_BASE(ValueError)
+    MP_DEFINE_EXCEPTION(UnicodeError, ValueError)
+    //TODO: Implement more UnicodeError subclasses which take arguments
+#endif
   /*
   MP_DEFINE_EXCEPTION(Warning, Exception)
     MP_DEFINE_EXCEPTION_BASE(Warning)
@@ -294,7 +306,7 @@ mp_obj_t mp_obj_new_exception_msg_varg(const mp_obj_type_t *exc_type, const char
         // Unfortunately, we won't be able to format the string...
         o = &MP_STATE_VM(mp_emergency_exception_obj);
         o->base.type = exc_type;
-        o->traceback = MP_OBJ_NULL;
+        o->traceback_data = NULL;
         o->args = mp_const_empty_tuple;
 
 #if MICROPY_ENABLE_EMERGENCY_EXCEPTION_BUF
@@ -313,13 +325,17 @@ mp_obj_t mp_obj_new_exception_msg_varg(const mp_obj_type_t *exc_type, const char
             uint max_len = MP_STATE_VM(mp_emergency_exception_buf) + mp_emergency_exception_buf_size
                          - str_data;
 
+            vstr_t vstr;
+            vstr_init_fixed_buf(&vstr, max_len, (char *)str_data);
+
             va_list ap;
             va_start(ap, fmt);
-            str->len = vsnprintf((char *)str_data, max_len, fmt, ap);
+            vstr_vprintf(&vstr, fmt, ap);
             va_end(ap);
 
             str->base.type = &mp_type_str;
             str->hash = qstr_compute_hash(str_data, str->len);
+            str->len = vstr.len;
             str->data = str_data;
 
             o->args = tuple;
@@ -328,36 +344,36 @@ mp_obj_t mp_obj_new_exception_msg_varg(const mp_obj_type_t *exc_type, const char
             offset += sizeof(void *) - 1;
             offset &= ~(sizeof(void *) - 1);
 
-            if ((mp_emergency_exception_buf_size - offset) > (sizeof(mp_obj_list_t) + sizeof(mp_obj_t) * 3)) {
+            if ((mp_emergency_exception_buf_size - offset) > (sizeof(mp_uint_t) * 3)) {
                 // We have room to store some traceback.
-                mp_obj_list_t *list = (mp_obj_list_t *)((byte *)MP_STATE_VM(mp_emergency_exception_buf) + offset);
-                list->base.type = &mp_type_list;
-                list->items = (mp_obj_t)&list[1];
-                list->alloc = (MP_STATE_VM(mp_emergency_exception_buf) + mp_emergency_exception_buf_size - (byte *)list->items) / sizeof(list->items[0]);
-                list->len = 0;
-
-                o->traceback = list;
+                o->traceback_data = (mp_uint_t*)((byte *)MP_STATE_VM(mp_emergency_exception_buf) + offset);
+                o->traceback_alloc = (MP_STATE_VM(mp_emergency_exception_buf) + mp_emergency_exception_buf_size - (byte *)o->traceback_data) / sizeof(o->traceback_data[0]);
+                o->traceback_len = 0;
             }
         }
 #endif // MICROPY_ENABLE_EMERGENCY_EXCEPTION_BUF
     } else {
         o->base.type = exc_type;
-        o->traceback = MP_OBJ_NULL;
+        o->traceback_data = NULL;
         o->args = mp_obj_new_tuple(1, NULL);
 
         if (fmt == NULL) {
             // no message
             assert(0);
         } else {
-            // render exception message and store as .args[0]
-            // TODO: optimize bufferbloat
-            vstr_t vstr;
-            vstr_init(&vstr, 16);
-            va_list ap;
-            va_start(ap, fmt);
-            vstr_vprintf(&vstr, fmt, ap);
-            va_end(ap);
-            o->args->items[0] = mp_obj_new_str_from_vstr(&mp_type_str, &vstr);
+            if (strchr(fmt, '%') == NULL) {
+                // no formatting substitutions, avoid allocating vstr.
+                o->args->items[0] = mp_obj_new_str(fmt, strlen(fmt), false);
+            } else {
+                // render exception message and store as .args[0]
+                va_list ap;
+                vstr_t vstr;
+                vstr_init(&vstr, 16);
+                va_start(ap, fmt);
+                vstr_vprintf(&vstr, fmt, ap);
+                va_end(ap);
+                o->args->items[0] = mp_obj_new_str_from_vstr(&mp_type_str, &vstr);
+            }
         }
     }
 
@@ -408,46 +424,47 @@ void mp_obj_exception_clear_traceback(mp_obj_t self_in) {
     GET_NATIVE_EXCEPTION(self, self_in);
     // just set the traceback to the null object
     // we don't want to call any memory management functions here
-    self->traceback = MP_OBJ_NULL;
+    self->traceback_data = NULL;
 }
 
 void mp_obj_exception_add_traceback(mp_obj_t self_in, qstr file, mp_uint_t line, qstr block) {
     GET_NATIVE_EXCEPTION(self, self_in);
 
-    #if MICROPY_ENABLE_GC
-    if (gc_is_locked()) {
-        if (self->traceback == MP_OBJ_NULL) {
-            // We can't allocate any memory, and no memory has been
-            // pre-allocated, so there is nothing else we can do.
-            return;
-        }
-        mp_obj_list_t *list = self->traceback;
-        if (list->alloc <= (list->len + 3)) {
-            // There is some preallocated memory, but not enough to store an
-            // entire record.
-            return;
-        }
-    }
-    #endif
+    // append this traceback info to traceback data
+    // if memory allocation fails (eg because gc is locked), just return
 
-    // for traceback, we are just using the list object for convenience, it's not really a list of Python objects
-    if (self->traceback == MP_OBJ_NULL) {
-        self->traceback = mp_obj_new_list(0, NULL);
+    if (self->traceback_data == NULL) {
+        self->traceback_data = m_new_maybe(mp_uint_t, 3);
+        if (self->traceback_data == NULL) {
+            return;
+        }
+        self->traceback_alloc = 3;
+        self->traceback_len = 0;
+    } else if (self->traceback_len + 3 > self->traceback_alloc) {
+        // be conservative with growing traceback data
+        mp_uint_t *tb_data = m_renew_maybe(mp_uint_t, self->traceback_data, self->traceback_alloc, self->traceback_alloc + 3);
+        if (tb_data == NULL) {
+            return;
+        }
+        self->traceback_data = tb_data;
+        self->traceback_alloc += 3;
     }
-    mp_obj_list_append(self->traceback, (mp_obj_t)(mp_uint_t)file);
-    mp_obj_list_append(self->traceback, (mp_obj_t)(mp_uint_t)line);
-    mp_obj_list_append(self->traceback, (mp_obj_t)(mp_uint_t)block);
+
+    mp_uint_t *tb_data = &self->traceback_data[self->traceback_len];
+    self->traceback_len += 3;
+    tb_data[0] = (mp_uint_t)file;
+    tb_data[1] = (mp_uint_t)line;
+    tb_data[2] = (mp_uint_t)block;
 }
 
 void mp_obj_exception_get_traceback(mp_obj_t self_in, mp_uint_t *n, mp_uint_t **values) {
     GET_NATIVE_EXCEPTION(self, self_in);
 
-    if (self->traceback == MP_OBJ_NULL) {
+    if (self->traceback_data == NULL) {
         *n = 0;
         *values = NULL;
     } else {
-        mp_uint_t n2;
-        mp_obj_list_get(self->traceback, &n2, (mp_obj_t**)values);
-        *n = n2;
+        *n = self->traceback_len;
+        *values = self->traceback_data;
     }
 }
