@@ -28,7 +28,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
-#include <math.h>
 
 #include "py/nlr.h"
 #include "py/parsenum.h"
@@ -37,30 +36,25 @@
 
 #if MICROPY_PY_BUILTINS_FLOAT
 
-#if MICROPY_FLOAT_IMPL == MICROPY_FLOAT_IMPL_FLOAT
+#include <math.h>
 #include "py/formatfloat.h"
-#endif
 
-STATIC void float_print(void (*print)(void *env, const char *fmt, ...), void *env, mp_obj_t o_in, mp_print_kind_t kind) {
+STATIC void float_print(const mp_print_t *print, mp_obj_t o_in, mp_print_kind_t kind) {
     (void)kind;
     mp_obj_float_t *o = o_in;
 #if MICROPY_FLOAT_IMPL == MICROPY_FLOAT_IMPL_FLOAT
     char buf[16];
-    mp_format_float(o->value, buf, sizeof(buf), 'g', 7, '\0');
-    print(env, "%s", buf);
-    if (strchr(buf, '.') == NULL && strchr(buf, 'e') == NULL) {
-        // Python floats always have decimal point
-        print(env, ".0");
-    }
+    const int precision = 7;
 #else
     char buf[32];
-    sprintf(buf, "%.16g", (double) o->value);
-    print(env, buf);
-    if (strchr(buf, '.') == NULL && strchr(buf, 'e') == NULL) {
-        // Python floats always have decimal point
-        print(env, ".0");
-    }
+    const int precision = 16;
 #endif
+    mp_format_float(o->value, buf, sizeof(buf), 'g', precision, '\0');
+    mp_print_str(print, buf);
+    if (strchr(buf, '.') == NULL && strchr(buf, 'e') == NULL && strchr(buf, 'n') == NULL) {
+        // Python floats always have decimal point (unless inf or nan)
+        mp_print_str(print, ".0");
+    }
 }
 
 STATIC mp_obj_t float_make_new(mp_obj_t type_in, mp_uint_t n_args, mp_uint_t n_kw, const mp_obj_t *args) {
@@ -132,6 +126,41 @@ mp_float_t mp_obj_float_get(mp_obj_t self_in) {
     return self->value;
 }
 
+STATIC void mp_obj_float_divmod(mp_float_t *x, mp_float_t *y) {
+    // logic here follows that of CPython
+    // https://docs.python.org/3/reference/expressions.html#binary-arithmetic-operations
+    // x == (x//y)*y + (x%y)
+    // divmod(x, y) == (x//y, x%y)
+    mp_float_t mod = MICROPY_FLOAT_C_FUN(fmod)(*x, *y);
+    mp_float_t div = (*x - mod) / *y;
+
+    // Python specs require that mod has same sign as second operand
+    if (mod == 0.0) {
+        mod = MICROPY_FLOAT_C_FUN(copysign)(0.0, *y);
+    } else {
+        if ((mod < 0.0) != (*y < 0.0)) {
+            mod += *y;
+            div -= 1.0;
+        }
+    }
+
+    mp_float_t floordiv;
+    if (div == 0.0) {
+        // if division is zero, take the correct sign of zero
+        floordiv = MICROPY_FLOAT_C_FUN(copysign)(0.0, *x / *y);
+    } else {
+        // Python specs require that x == (x//y)*y + (x%y)
+        floordiv = MICROPY_FLOAT_C_FUN(floor)(div);
+        if (div - floordiv > 0.5) {
+            floordiv += 1.0;
+        }
+    }
+
+    // return results
+    *x = floordiv;
+    *y = mod;
+}
+
 mp_obj_t mp_obj_float_binary_op(mp_uint_t op, mp_float_t lhs_val, mp_obj_t rhs_in) {
     mp_float_t rhs_val = mp_obj_get_float(rhs_in); // can be any type, this function will convert to float (if possible)
     switch (op) {
@@ -176,6 +205,17 @@ mp_obj_t mp_obj_float_binary_op(mp_uint_t op, mp_float_t lhs_val, mp_obj_t rhs_i
             break;
         case MP_BINARY_OP_POWER:
         case MP_BINARY_OP_INPLACE_POWER: lhs_val = MICROPY_FLOAT_C_FUN(pow)(lhs_val, rhs_val); break;
+        case MP_BINARY_OP_DIVMOD: {
+            if (rhs_val == 0) {
+                goto zero_division_error;
+            }
+            mp_obj_float_divmod(&lhs_val, &rhs_val);
+            mp_obj_t tuple[2] = {
+                mp_obj_new_float(lhs_val),
+                mp_obj_new_float(rhs_val),
+            };
+            return mp_obj_new_tuple(2, tuple);
+        }
         case MP_BINARY_OP_LESS: return MP_BOOL(lhs_val < rhs_val);
         case MP_BINARY_OP_MORE: return MP_BOOL(lhs_val > rhs_val);
         case MP_BINARY_OP_EQUAL: return MP_BOOL(lhs_val == rhs_val);
@@ -188,39 +228,4 @@ mp_obj_t mp_obj_float_binary_op(mp_uint_t op, mp_float_t lhs_val, mp_obj_t rhs_i
     return mp_obj_new_float(lhs_val);
 }
 
-void mp_obj_float_divmod(mp_float_t *x, mp_float_t *y) {
-    // logic here follows that of CPython
-    // https://docs.python.org/3/reference/expressions.html#binary-arithmetic-operations
-    // x == (x//y)*y + (x%y)
-    // divmod(x, y) == (x//y, x%y)
-    mp_float_t mod = MICROPY_FLOAT_C_FUN(fmod)(*x, *y);
-    mp_float_t div = (*x - mod) / *y;
-
-    // Python specs require that mod has same sign as second operand
-    if (mod == 0.0) {
-        mod = MICROPY_FLOAT_C_FUN(copysign)(0.0, *y);
-    } else {
-        if ((mod < 0.0) != (*y < 0.0)) {
-            mod += *y;
-            div -= 1.0;
-        }
-    }
-
-    mp_float_t floordiv;
-    if (div == 0.0) {
-        // if division is zero, take the correct sign of zero
-        floordiv = MICROPY_FLOAT_C_FUN(copysign)(0.0, *x / *y);
-    } else {
-        // Python specs require that x == (x//y)*y + (x%y)
-        floordiv = MICROPY_FLOAT_C_FUN(floor)(div);
-        if (div - floordiv > 0.5) {
-            floordiv += 1.0;
-        }
-    }
-
-    // return results
-    *x = floordiv;
-    *y = mod;
-}
-
 #endif // MICROPY_PY_BUILTINS_FLOAT
