@@ -66,6 +66,11 @@ Q(__add__)
 Q(__sub__)
 Q(__repr__)
 Q(__str__)
+#if MICROPY_PY_DESCRIPTORS
+Q(__get__)
+Q(__set__)
+Q(__delete__)
+#endif
 Q(__getattr__)
 Q(__del__)
 Q(__call__)
@@ -76,7 +81,14 @@ Q(__le__)
 Q(__ge__)
 Q(__reversed__)
 #if MICROPY_PY_ALL_SPECIAL_METHODS
+Q(__mul__)
+Q(__truediv__)
+Q(__floordiv__)
 Q(__iadd__)
+Q(__isub__)
+Q(__invert__)
+Q(__neg__)
+Q(__pos__)
 #endif
 
 Q(micropython)
@@ -103,6 +115,9 @@ Q(builtins)
 
 Q(Ellipsis)
 Q(StopIteration)
+#if MICROPY_PY_BUILTINS_NOTIMPLEMENTED
+Q(NotImplemented)
+#endif
 
 Q(BaseException)
 Q(ArithmeticError)
@@ -132,7 +147,13 @@ Q(SystemExit)
 Q(TypeError)
 Q(UnboundLocalError)
 Q(ValueError)
+#if MICROPY_EMIT_NATIVE
+Q(ViperTypeError)
+#endif
 Q(ZeroDivisionError)
+#if MICROPY_PY_BUILTINS_STR_UNICODE
+Q(UnicodeError)
+#endif
 
 Q(None)
 Q(False)
@@ -141,6 +162,10 @@ Q(object)
 
 Q(NoneType)
 
+#if MICROPY_PY_COLLECTIONS_ORDEREDDICT
+Q(OrderedDict)
+#endif
+
 Q(abs)
 Q(all)
 Q(any)
@@ -159,9 +184,6 @@ Q(memoryview)
 #endif
 Q(bytes)
 Q(callable)
-#if MICROPY_PY_STRUCT
-Q(calcsize)
-#endif
 Q(chr)
 Q(classmethod)
 Q(_collections)
@@ -185,6 +207,7 @@ Q(float)
 #endif
 Q(from_bytes)
 Q(getattr)
+Q(setattr)
 Q(globals)
 Q(hasattr)
 Q(hash)
@@ -237,6 +260,11 @@ Q(single)
 Q(sep)
 Q(end)
 
+#if MICROPY_PY_BUILTINS_RANGE_ATTRS
+Q(step)
+Q(stop)
+#endif
+
 Q(clear)
 Q(copy)
 Q(fromkeys)
@@ -275,6 +303,11 @@ Q(find)
 Q(rfind)
 Q(rindex)
 Q(split)
+#if MICROPY_PY_BUILTINS_STR_SPLITLINES
+Q(splitlines)
+Q(keepends)
+Q(\n)
+#endif
 Q(rsplit)
 Q(startswith)
 Q(endswith)
@@ -358,11 +391,13 @@ Q(frexp)
 Q(ldexp)
 Q(degrees)
 Q(radians)
+#if MICROPY_PY_MATH_SPECIAL_FUNCTIONS
 Q(erf)
 Q(erfc)
 Q(gamma)
 Q(lgamma)
 #endif
+#endif
 
 #if MICROPY_PY_CMATH
 Q(cmath)
@@ -384,6 +419,7 @@ Q(qstr_info)
 #if MICROPY_ENABLE_EMERGENCY_EXCEPTION_BUF && (MICROPY_EMERGENCY_EXCEPTION_BUF_SIZE == 0)
 Q(alloc_emergency_exception_buf)
 #endif
+Q(maximum recursion depth exceeded)
 
 Q(<module>)
 Q(<lambda>)
@@ -412,22 +448,34 @@ Q(platform)
 Q(stdin)
 Q(stdout)
 Q(stderr)
+#if MICROPY_PY_SYS_STDIO_BUFFER
+Q(buffer)
+#endif
 Q(version)
 Q(version_info)
+#if MICROPY_PY_ATTRTUPLE
+Q(name)
+#endif
+Q(implementation)
 #if MICROPY_PY_SYS_MAXSIZE
 Q(maxsize)
 #endif
+#if MICROPY_PY_SYS_EXC_INFO
+Q(exc_info)
+#endif
 Q(print_exception)
 #endif
 
 #if MICROPY_PY_STRUCT
-Q(struct)
+Q(ustruct)
 Q(pack)
 Q(unpack)
+Q(calcsize)
 #endif
 
 #if MICROPY_PY_UCTYPES
 Q(uctypes)
+Q(struct)
 Q(sizeof)
 Q(addressof)
 Q(bytes_at)
@@ -532,11 +580,19 @@ Q(heapify)
 Q(uhashlib)
 Q(update)
 Q(digest)
-Q(hexdigest)
 Q(sha256)
 #endif
 
 #if MICROPY_PY_UBINASCII
 Q(ubinascii)
 Q(hexlify)
+Q(unhexlify)
+#endif
+
+#if MICROPY_PY_MACHINE
+Q(machine)
+Q(mem)
+Q(mem8)
+Q(mem16)
+Q(mem32)
 #endif
