@@ -50,6 +50,7 @@
 typedef enum {
     MP_UNARY_OP_BOOL, // __bool__
     MP_UNARY_OP_LEN, // __len__
+    MP_UNARY_OP_HASH, // __hash__; must return a small int
     MP_UNARY_OP_POSITIVE,
     MP_UNARY_OP_NEGATIVE,
     MP_UNARY_OP_INVERT,
@@ -73,29 +74,30 @@ typedef enum {
 
     MP_BINARY_OP_MODULO,
     MP_BINARY_OP_POWER,
+    MP_BINARY_OP_DIVMOD, // not emitted by the compiler but supported by the runtime
     MP_BINARY_OP_INPLACE_OR,
     MP_BINARY_OP_INPLACE_XOR,
-    MP_BINARY_OP_INPLACE_AND,
 
+    MP_BINARY_OP_INPLACE_AND,
     MP_BINARY_OP_INPLACE_LSHIFT,
     MP_BINARY_OP_INPLACE_RSHIFT,
     MP_BINARY_OP_INPLACE_ADD,
     MP_BINARY_OP_INPLACE_SUBTRACT,
-    MP_BINARY_OP_INPLACE_MULTIPLY,
 
+    MP_BINARY_OP_INPLACE_MULTIPLY,
     MP_BINARY_OP_INPLACE_FLOOR_DIVIDE,
     MP_BINARY_OP_INPLACE_TRUE_DIVIDE,
     MP_BINARY_OP_INPLACE_MODULO,
     MP_BINARY_OP_INPLACE_POWER,
+
     // these should return a bool
     MP_BINARY_OP_LESS,
-
     MP_BINARY_OP_MORE,
     MP_BINARY_OP_EQUAL,
     MP_BINARY_OP_LESS_EQUAL,
     MP_BINARY_OP_MORE_EQUAL,
-    MP_BINARY_OP_NOT_EQUAL,
 
+    MP_BINARY_OP_NOT_EQUAL,
     MP_BINARY_OP_IN,
     MP_BINARY_OP_IS,
     MP_BINARY_OP_EXCEPTION_MATCH,
@@ -133,6 +135,7 @@ typedef enum {
     MP_F_MAKE_FUNCTION_FROM_RAW_CODE,
     MP_F_NATIVE_CALL_FUNCTION_N_KW,
     MP_F_CALL_METHOD_N_KW,
+    MP_F_CALL_METHOD_N_KW_VAR,
     MP_F_GETITER,
     MP_F_ITERNEXT,
     MP_F_NLR_PUSH,
@@ -148,6 +151,9 @@ typedef enum {
     MP_F_UNPACK_EX,
     MP_F_DELETE_NAME,
     MP_F_DELETE_GLOBAL,
+    MP_F_NEW_CELL,
+    MP_F_MAKE_CLOSURE_FROM_RAW_CODE,
+    MP_F_SETUP_CODE_STATE,
     MP_F_NUMBER_OF,
 } mp_fun_kind_t;
 
