@@ -35,14 +35,17 @@
 #define FL_ALPHA (0x08)
 #define FL_UPPER (0x10)
 #define FL_LOWER (0x20)
+#define FL_XDIGIT (0x40)
 
 // shorthand character attributes
 #define AT_PR (FL_PRINT)
 #define AT_SP (FL_SPACE | FL_PRINT)
-#define AT_DI (FL_DIGIT | FL_PRINT)
+#define AT_DI (FL_DIGIT | FL_PRINT | FL_XDIGIT)
 #define AT_AL (FL_ALPHA | FL_PRINT)
 #define AT_UP (FL_UPPER | FL_ALPHA | FL_PRINT)
 #define AT_LO (FL_LOWER | FL_ALPHA | FL_PRINT)
+#define AT_UX (FL_UPPER | FL_ALPHA | FL_PRINT | FL_XDIGIT)
+#define AT_LX (FL_LOWER | FL_ALPHA | FL_PRINT | FL_XDIGIT)
 
 // table of attributes for ascii characters
 STATIC const uint8_t attr[] = {
@@ -54,11 +57,11 @@ STATIC const uint8_t attr[] = {
     AT_PR, AT_PR, AT_PR, AT_PR, AT_PR, AT_PR, AT_PR, AT_PR,
     AT_DI, AT_DI, AT_DI, AT_DI, AT_DI, AT_DI, AT_DI, AT_DI,
     AT_DI, AT_DI, AT_PR, AT_PR, AT_PR, AT_PR, AT_PR, AT_PR,
-    AT_PR, AT_UP, AT_UP, AT_UP, AT_UP, AT_UP, AT_UP, AT_UP,
+    AT_PR, AT_UX, AT_UX, AT_UX, AT_UX, AT_UX, AT_UX, AT_UP,
     AT_UP, AT_UP, AT_UP, AT_UP, AT_UP, AT_UP, AT_UP, AT_UP,
     AT_UP, AT_UP, AT_UP, AT_UP, AT_UP, AT_UP, AT_UP, AT_UP,
     AT_UP, AT_UP, AT_UP, AT_PR, AT_PR, AT_PR, AT_PR, AT_PR,
-    AT_PR, AT_LO, AT_LO, AT_LO, AT_LO, AT_LO, AT_LO, AT_LO,
+    AT_PR, AT_LX, AT_LX, AT_LX, AT_LX, AT_LX, AT_LX, AT_LO,
     AT_LO, AT_LO, AT_LO, AT_LO, AT_LO, AT_LO, AT_LO, AT_LO,
     AT_LO, AT_LO, AT_LO, AT_LO, AT_LO, AT_LO, AT_LO, AT_LO,
     AT_LO, AT_LO, AT_LO, AT_PR, AT_PR, AT_PR, AT_PR, 0
@@ -107,8 +110,7 @@ mp_uint_t utf8_ptr_to_index(const byte *s, const byte *ptr) {
 }
 
 // TODO: Rename to str_charlen
-mp_uint_t unichar_charlen(const char *str, mp_uint_t len)
-{
+mp_uint_t unichar_charlen(const char *str, mp_uint_t len) {
 #if MICROPY_PY_BUILTINS_STR_UNICODE
     mp_uint_t charlen = 0;
     for (const char *top = str + len; str < top; ++str) {
@@ -140,7 +142,7 @@ bool unichar_isdigit(unichar c) {
 }
 
 bool unichar_isxdigit(unichar c) {
-    return unichar_isdigit(c) || (c >= 'a' && c <= 'f') || (c >= 'A' && c <= 'F');
+    return c < 128 && (attr[c] & FL_XDIGIT) != 0;
 }
 
 /*
@@ -170,3 +172,13 @@ unichar unichar_toupper(unichar c) {
     }
     return c;
 }
+
+mp_uint_t unichar_xdigit_value(unichar c) {
+    // c is assumed to be hex digit
+    mp_uint_t n = c - '0';
+    if (n > 9) {
+        n &= ~('a' - 'A');
+        n -= ('A' - ('9' + 1));
+    }
+    return n;
+}
