@@ -35,6 +35,8 @@
 #include "py/stackctrl.h"
 #include "py/gc.h"
 
+#include "lib/fatfs/ff.h"
+
 #include "systick.h"
 #include "pendsv.h"
 #include "gccollect.h"
@@ -52,13 +54,13 @@
 #include "rtc.h"
 #include "storage.h"
 #include "sdcard.h"
-#include "ff.h"
 #include "rng.h"
 #include "accel.h"
 #include "servo.h"
 #include "dac.h"
-#include "pybstdio.h"
+#include "can.h"
 #include "modnetwork.h"
+#include MICROPY_HAL_H
 
 void SystemClock_Config(void);
 
@@ -86,8 +88,8 @@ void NORETURN __fatal_error(const char *msg) {
     led_state(2, 1);
     led_state(3, 1);
     led_state(4, 1);
-    stdout_tx_strn("\nFATAL ERROR:\n", 14);
-    stdout_tx_strn(msg, strlen(msg));
+    mp_hal_stdout_tx_strn("\nFATAL ERROR:\n", 14);
+    mp_hal_stdout_tx_strn(msg, strlen(msg));
     for (uint i = 0;;) {
         led_toggle(((i++) & 3) + 1);
         for (volatile uint delay = 0; delay < 10000000; delay++) {
@@ -120,14 +122,6 @@ STATIC mp_obj_t pyb_main(mp_obj_t main) {
 }
 MP_DEFINE_CONST_FUN_OBJ_1(pyb_main_obj, pyb_main);
 
-STATIC mp_obj_t pyb_usb_mode(mp_obj_t usb_mode) {
-    if (MP_OBJ_IS_STR(usb_mode)) {
-        MP_STATE_PORT(pyb_config_usb_mode) = usb_mode;
-    }
-    return mp_const_none;
-}
-MP_DEFINE_CONST_FUN_OBJ_1(pyb_usb_mode_obj, pyb_usb_mode);
-
 static const char fresh_boot_py[] =
 "# boot.py -- run on boot-up\r\n"
 "# can run arbitrary Python, but best to keep it minimal\r\n"
@@ -309,6 +303,11 @@ int main(void) {
     switch_init0();
 #endif
 
+#if defined(USE_DEVICE_MODE)
+    // default to internal flash being the usb medium
+    pyb_usb_storage_medium = PYB_USB_STORAGE_MEDIUM_FLASH;
+#endif
+
     int first_soft_reset = true;
 
 soft_reset:
@@ -378,20 +377,6 @@ soft_reset:
     mp_obj_list_append(mp_sys_path, MP_OBJ_NEW_QSTR(MP_QSTR__slash_flash_slash_lib));
     mp_obj_list_init(mp_sys_argv, 0);
 
-    // Change #if 0 to #if 1 if you want REPL on UART_6 (or another uart)
-    // as well as on USB VCP
-#if 0
-    {
-        mp_obj_t args[2] = {
-            MP_OBJ_NEW_SMALL_INT(PYB_UART_6),
-            MP_OBJ_NEW_SMALL_INT(115200),
-        };
-        pyb_stdio_uart = pyb_uart_type.make_new((mp_obj_t)&pyb_uart_type, MP_ARRAY_SIZE(args), 0, args);
-    }
-#else
-    MP_STATE_PORT(pyb_stdio_uart) = NULL;
-#endif
-
     // Initialise low-level sub-systems.  Here we need to very basic things like
     // zeroing out memory and resetting any of the sub-systems.  Following this
     // we can run Python scripts (eg boot.py), but anything that is configurable
@@ -403,6 +388,24 @@ soft_reset:
     timer_init0();
     uart_init0();
 
+    // Change #if 0 to #if 1 if you want REPL on UART_6 (or another uart)
+    // as well as on USB VCP
+#if 0
+    {
+        mp_obj_t args[2] = {
+            MP_OBJ_NEW_SMALL_INT(PYB_UART_6),
+            MP_OBJ_NEW_SMALL_INT(115200),
+        };
+        MP_STATE_PORT(pyb_stdio_uart) = pyb_uart_type.make_new((mp_obj_t)&pyb_uart_type, MP_ARRAY_SIZE(args), 0, args);
+    }
+#else
+    MP_STATE_PORT(pyb_stdio_uart) = NULL;
+#endif
+
+#if MICROPY_HW_ENABLE_CAN
+    can_init0();
+#endif
+
 #if MICROPY_HW_ENABLE_RNG
     rng_init0();
 #endif
@@ -415,10 +418,6 @@ soft_reset:
     // Create it if needed, mount in on /flash, and set it as current dir.
     init_flash_fs(reset_mode);
 
-#if defined(USE_DEVICE_MODE)
-    usb_storage_medium_t usb_medium = USB_STORAGE_MEDIUM_FLASH;
-#endif
-
 #if MICROPY_HW_HAS_SDCARD
     // if an SD card is present then mount it on /sd/
     if (sdcard_is_present()) {
@@ -426,9 +425,6 @@ soft_reset:
         if (res != FR_OK) {
             printf("[SD] could not mount SD card\n");
         } else {
-            // use SD card as current directory
-            f_chdrive("/sd");
-
             // TODO these should go before the /flash entries in the path
             mp_obj_list_append(mp_sys_path, MP_OBJ_NEW_QSTR(MP_QSTR__slash_sd));
             mp_obj_list_append(mp_sys_path, MP_OBJ_NEW_QSTR(MP_QSTR__slash_sd_slash_lib));
@@ -436,16 +432,24 @@ soft_reset:
             if (first_soft_reset) {
                 // use SD card as medium for the USB MSD
 #if defined(USE_DEVICE_MODE)
-                usb_medium = USB_STORAGE_MEDIUM_SDCARD;
+                pyb_usb_storage_medium = PYB_USB_STORAGE_MEDIUM_SDCARD;
 #endif
             }
+
+            #if defined(USE_DEVICE_MODE)
+            // only use SD card as current directory if that's what the USB medium is
+            if (pyb_usb_storage_medium == PYB_USB_STORAGE_MEDIUM_SDCARD)
+            #endif
+            {
+                // use SD card as current directory
+                f_chdrive("/sd");
+            }
         }
     }
 #endif
 
     // reset config variables; they should be set by boot.py
     MP_STATE_PORT(pyb_config_main) = MP_OBJ_NULL;
-    MP_STATE_PORT(pyb_config_usb_mode) = MP_OBJ_NULL;
 
     // run boot.py, if it exists
     // TODO perhaps have pyb.reboot([bootpy]) function to soft-reboot and execute custom boot.py
@@ -472,19 +476,11 @@ soft_reset:
     // or whose initialisation can be safely deferred until after running
     // boot.py.
 
-#if defined(USE_HOST_MODE)
-    // USB host
-    pyb_usb_host_init();
-#elif defined(USE_DEVICE_MODE)
-    // USB device
-    usb_device_mode_t usb_mode = USB_DEVICE_MODE_CDC_MSC;
-    // if we are not in reset_mode==1, this config variable will always be NULL
-    if (MP_STATE_PORT(pyb_config_usb_mode) != MP_OBJ_NULL) {
-        if (strcmp(mp_obj_str_get_str(MP_STATE_PORT(pyb_config_usb_mode)), "CDC+HID") == 0) {
-            usb_mode = USB_DEVICE_MODE_CDC_HID;
-        }
+#if defined(USE_DEVICE_MODE)
+    // init USB device to default setting if it was not already configured
+    if (!(pyb_usb_flags & PYB_USB_FLAG_USB_MODE_CALLED)) {
+        pyb_usb_dev_init(USBD_VID, USBD_PID_CDC_MSC, USBD_MODE_CDC_MSC, NULL);
     }
-    pyb_usb_dev_init(usb_mode, usb_medium);
 #endif
 
 #if MICROPY_HW_HAS_MMA7660
@@ -550,6 +546,9 @@ soft_reset_exit:
     printf("PYB: soft reboot\n");
     timer_deinit();
     uart_deinit();
+#if MICROPY_HW_ENABLE_CAN
+    can_deinit();
+#endif
 
     first_soft_reset = false;
     goto soft_reset;
