@@ -68,9 +68,9 @@ typedef struct _stm_mem_obj_t {
     uint32_t elem_size; // in bytes
 } stm_mem_obj_t;
 
-STATIC void stm_mem_print(void (*print)(void *env, const char *fmt, ...), void *env, mp_obj_t self_in, mp_print_kind_t kind) {
+STATIC void stm_mem_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
     stm_mem_obj_t *self = self_in;
-    print(env, "<%u-bit memory>", 8 * self->elem_size);
+    mp_printf(print, "<%u-bit memory>", 8 * self->elem_size);
 }
 
 STATIC mp_obj_t stm_mem_subscr(mp_obj_t self_in, mp_obj_t index, mp_obj_t value) {
