@@ -31,10 +31,12 @@
 #include "py/runtime.h"
 #include "pendsv.h"
 
-// Note: this can contain point to the heap but is not traced by GC.
-// This is okay because we only ever set it to mp_const_vcp_interrupt
-// which is in the root-pointer set.
-STATIC void *pendsv_object;
+// This variable is used to save the exception object between a ctrl-C and the
+// PENDSV call that actually raises the exception.  It must be non-static
+// otherwise gcc-5 optimises it away.  It can point to the heap but is not
+// traced by GC.  This is okay because we only ever set it to
+// mp_const_vcp_interrupt which is in the root-pointer set.
+void *pendsv_object;
 
 void pendsv_init(void) {
     // set PendSV interrupt at lowest priority
@@ -59,10 +61,6 @@ void pendsv_nlr_jump(void *o) {
     }
 }
 
-// since we play tricks with the stack, the compiler must not generate a
-// prelude for this function
-void pendsv_isr_handler(void) __attribute__((naked));
-
 void pendsv_isr_handler(void) {
     // re-jig the stack so that when we return from this interrupt handler
     // it returns instead to nlr_jump with argument pendsv_object
