@@ -41,7 +41,6 @@ Q(enable_irq)
 Q(stop)
 Q(standby)
 Q(main)
-Q(usb_mode)
 Q(sync)
 Q(gc)
 Q(repl_info)
@@ -53,9 +52,7 @@ Q(read)
 Q(readall)
 Q(readline)
 Q(write)
-Q(have_cdc)
 Q(repl_uart)
-Q(hid)
 Q(time)
 Q(rng)
 Q(SD)
@@ -95,17 +92,37 @@ Q(hashlib)
 Q(seek)
 Q(tell)
 
+// for USB configuration
+Q(usb_mode)
+Q(mode)
+Q(vid)
+Q(pid)
+Q(hid)
+Q(hid_mouse)
+Q(hid_keyboard)
+
 // for USB VCP class
 Q(USB_VCP)
 Q(setinterrupt)
+Q(isconnected)
+Q(have_cdc)
+Q(any)
 Q(send)
 Q(recv)
 Q(timeout)
 
+// for USB HID class
+Q(USB_HID)
+Q(any)
+Q(send)
+Q(recv)
+
 // for RTC class
 Q(RTC)
 Q(info)
 Q(datetime)
+Q(wakeup)
+Q(calibration)
 
 // for Pin class
 Q(Pin)
@@ -171,6 +188,7 @@ Q(any)
 Q(writechar)
 Q(readchar)
 Q(readinto)
+Q(sendbreak)
 Q(RTS)
 Q(CTS)
 
@@ -195,6 +213,8 @@ Q(params)
 Q(initfilterbanks)
 Q(clearfilter)
 Q(setfilter)
+Q(rxcallback)
+Q(rtr)
 Q(NORMAL)
 Q(LOOPBACK)
 Q(SILENT)
@@ -230,6 +250,9 @@ Q(OC_INACTIVE)
 Q(OC_TOGGLE)
 Q(OC_FORCED_ACTIVE)
 Q(OC_FORCED_INACTIVE)
+Q(ENC_A)
+Q(ENC_B)
+Q(ENC_AB)
 Q(HIGH)
 Q(LOW)
 Q(RISING)
@@ -346,10 +369,17 @@ Q(os)
 Q(/)
 Q(flash)
 Q(sd)
+Q(sysname)
+Q(nodename)
+Q(release)
+Q(version)
+Q(machine)
+Q(uname)
 Q(chdir)
 Q(getcwd)
 Q(listdir)
 Q(mkdir)
+Q(rename)
 Q(remove)
 Q(rmdir)
 Q(unlink)
@@ -548,6 +578,8 @@ Q(I2C_OAR2)
 Q(I2C_SR1)
 Q(I2C_SR2)
 Q(I2C_TRISE)
+Q(I2S2EXT)
+Q(I2S3EXT)
 Q(IWDG)
 Q(IWDG_KR)
 Q(IWDG_PR)
