@@ -71,10 +71,12 @@
 #include "stm32f4xx_hal.h"
 
 #include "py/obj.h"
+#include "pendsv.h"
 #include "extint.h"
 #include "timer.h"
 #include "uart.h"
 #include "storage.h"
+#include "can.h"
 
 extern void __fatal_error(const char*);
 extern PCD_HandleTypeDef pcd_handle;
@@ -161,7 +163,6 @@ void DebugMon_Handler(void) {
   * @retval None
   */
 void PendSV_Handler(void) {
-    extern void pendsv_isr_handler(void);
     pendsv_isr_handler();
 }
 
@@ -340,7 +341,8 @@ void TAMP_STAMP_IRQHandler(void) {
 }
 
 void RTC_WKUP_IRQHandler(void) {
-    Handle_EXTI_Irq(EXTI_RTC_WAKEUP);
+    RTC->ISR &= ~(1 << 10); // clear wakeup interrupt flag
+    Handle_EXTI_Irq(EXTI_RTC_WAKEUP); // clear EXTI flag and execute optional callback
 }
 
 void TIM1_BRK_TIM9_IRQHandler(void) {
@@ -411,6 +413,28 @@ void UART4_IRQHandler(void) {
     uart_irq_handler(4);
 }
 
+void UART5_IRQHandler(void) {
+    uart_irq_handler(5);
+}
+
 void USART6_IRQHandler(void) {
     uart_irq_handler(6);
 }
+
+#if MICROPY_HW_ENABLE_CAN
+void CAN1_RX0_IRQHandler(void) {
+    can_rx_irq_handler(PYB_CAN_1, CAN_FIFO0);
+}
+
+void CAN1_RX1_IRQHandler(void) {
+    can_rx_irq_handler(PYB_CAN_1, CAN_FIFO1);
+}
+
+void CAN2_RX0_IRQHandler(void) {
+    can_rx_irq_handler(PYB_CAN_2, CAN_FIFO0);
+}
+
+void CAN2_RX1_IRQHandler(void) {
+    can_rx_irq_handler(PYB_CAN_2, CAN_FIFO1);
+}
+#endif // MICROPY_HW_ENABLE_CAN
