@@ -35,7 +35,7 @@
 #include "py/nlr.h"
 #include "py/runtime.h"
 #include "py/gc.h"
-#include "py/pfenv.h"
+#include MICROPY_HAL_H
 #include "timer.h"
 #include "servo.h"
 #include "pin.h"
@@ -96,6 +96,9 @@ typedef enum {
     CHANNEL_MODE_OC_FORCED_ACTIVE,
     CHANNEL_MODE_OC_FORCED_INACTIVE,
     CHANNEL_MODE_IC,
+    CHANNEL_MODE_ENC_A,
+    CHANNEL_MODE_ENC_B,
+    CHANNEL_MODE_ENC_AB,
 } pyb_channel_mode;
 
 STATIC const struct {
@@ -111,6 +114,9 @@ STATIC const struct {
     { MP_QSTR_OC_FORCED_ACTIVE,   TIM_OCMODE_FORCED_ACTIVE },
     { MP_QSTR_OC_FORCED_INACTIVE, TIM_OCMODE_FORCED_INACTIVE },
     { MP_QSTR_IC,                 0 },
+    { MP_QSTR_ENC_A,              TIM_ENCODERMODE_TI1 },
+    { MP_QSTR_ENC_B,              TIM_ENCODERMODE_TI2 },
+    { MP_QSTR_ENC_AB,             TIM_ENCODERMODE_TI12 },
 };
 
 typedef struct _pyb_timer_channel_obj_t {
@@ -217,6 +223,7 @@ void timer_tim5_init(void) {
     HAL_TIM_PWM_Init(&TIM5_Handle);
 }
 
+#if defined(TIM6)
 // Init TIM6 with a counter-overflow at the given frequency (given in Hz)
 // TIM6 is used by the DAC and ADC for auto sampling at a given frequency
 // This function inits but does not start the timer
@@ -241,6 +248,7 @@ void timer_tim6_init(uint freq) {
     TIM6_Handle.Init.CounterMode = TIM_COUNTERMODE_UP; // unused for TIM6
     HAL_TIM_Base_Init(&TIM6_Handle);
 }
+#endif
 
 // Interrupt dispatch
 void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim) {
@@ -462,17 +470,17 @@ STATIC void config_deadtime(pyb_timer_obj_t *self, mp_int_t ticks) {
     HAL_TIMEx_ConfigBreakDeadTime(&self->tim, &deadTimeConfig);
 }
 
-STATIC void pyb_timer_print(void (*print)(void *env, const char *fmt, ...), void *env, mp_obj_t self_in, mp_print_kind_t kind) {
+STATIC void pyb_timer_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
     pyb_timer_obj_t *self = self_in;
 
     if (self->tim.State == HAL_TIM_STATE_RESET) {
-        print(env, "Timer(%u)", self->tim_id);
+        mp_printf(print, "Timer(%u)", self->tim_id);
     } else {
         uint32_t prescaler = self->tim.Instance->PSC & 0xffff;
         uint32_t period = __HAL_TIM_GetAutoreload(&self->tim) & TIMER_CNT_MASK(self);
         // for efficiency, we compute and print freq as an int (not a float)
         uint32_t freq = timer_get_source_freq(self->tim_id) / ((prescaler + 1) * (period + 1));
-        print(env, "Timer(%u, freq=%u, prescaler=%u, period=%u, mode=%s, div=%u",
+        mp_printf(print, "Timer(%u, freq=%u, prescaler=%u, period=%u, mode=%s, div=%u",
             self->tim_id,
             freq,
             prescaler,
@@ -482,9 +490,10 @@ STATIC void pyb_timer_print(void (*print)(void *env, const char *fmt, ...), void
             self->tim.Init.ClockDivision == TIM_CLOCKDIVISION_DIV4 ? 4 :
             self->tim.Init.ClockDivision == TIM_CLOCKDIVISION_DIV2 ? 2 : 1);
         if (IS_TIM_ADVANCED_INSTANCE(self->tim.Instance)) {
-            print(env, ", deadtime=%u", compute_ticks_from_dtg(self->tim.Instance->BDTR & TIM_BDTR_DTG));
+            mp_printf(print, ", deadtime=%u",
+                compute_ticks_from_dtg(self->tim.Instance->BDTR & TIM_BDTR_DTG));
         }
-        print(env, ")");
+        mp_print_str(print, ")");
     }
 }
 
@@ -580,15 +589,27 @@ STATIC mp_obj_t pyb_timer_init_helper(pyb_timer_obj_t *self, mp_uint_t n_args, c
         case 3: __TIM3_CLK_ENABLE(); break;
         case 4: __TIM4_CLK_ENABLE(); break;
         case 5: __TIM5_CLK_ENABLE(); break;
+        #if defined(TIM6)
         case 6: __TIM6_CLK_ENABLE(); break;
+        #endif
+        #if defined(TIM7)
         case 7: __TIM7_CLK_ENABLE(); break;
+        #endif
+        #if defined(TIM8)
         case 8: __TIM8_CLK_ENABLE(); break;
+        #endif
         case 9: __TIM9_CLK_ENABLE(); break;
         case 10: __TIM10_CLK_ENABLE(); break;
         case 11: __TIM11_CLK_ENABLE(); break;
+        #if defined(TIM12)
         case 12: __TIM12_CLK_ENABLE(); break;
+        #endif
+        #if defined(TIM13)
         case 13: __TIM13_CLK_ENABLE(); break;
+        #endif
+        #if defined(TIM14)
         case 14: __TIM14_CLK_ENABLE(); break;
+        #endif
     }
 
     // set IRQ priority (if not a special timer)
@@ -636,18 +657,35 @@ STATIC mp_obj_t pyb_timer_make_new(mp_obj_t type_in, mp_uint_t n_args, mp_uint_t
         case 3: nlr_raise(mp_obj_new_exception_msg(&mp_type_ValueError, "Timer 3 is for internal use only")); // TIM3 used for low-level stuff; go via regs if necessary
         case 4: tim->tim.Instance = TIM4; tim->irqn = TIM4_IRQn; break;
         case 5: tim->tim.Instance = TIM5; tim->irqn = TIM5_IRQn; tim->is_32bit = true; break;
+        #if defined(TIM6)
         case 6: tim->tim.Instance = TIM6; tim->irqn = TIM6_DAC_IRQn; break;
+        #endif
+        #if defined(TIM7)
         case 7: tim->tim.Instance = TIM7; tim->irqn = TIM7_IRQn; break;
+        #endif
+        #if defined(TIM8)
         case 8: tim->tim.Instance = TIM8; tim->irqn = TIM8_UP_TIM13_IRQn; break;
+        #endif
         case 9: tim->tim.Instance = TIM9; tim->irqn = TIM1_BRK_TIM9_IRQn; break;
         case 10: tim->tim.Instance = TIM10; tim->irqn = TIM1_UP_TIM10_IRQn; break;
         case 11: tim->tim.Instance = TIM11; tim->irqn = TIM1_TRG_COM_TIM11_IRQn; break;
+        #if defined(TIM12)
         case 12: tim->tim.Instance = TIM12; tim->irqn = TIM8_BRK_TIM12_IRQn; break;
+        #endif
+        #if defined(TIM13)
         case 13: tim->tim.Instance = TIM13; tim->irqn = TIM8_UP_TIM13_IRQn; break;
+        #endif
+        #if defined(TIM14)
         case 14: tim->tim.Instance = TIM14; tim->irqn = TIM8_TRG_COM_TIM14_IRQn; break;
+        #endif
         default: nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_ValueError, "Timer %d does not exist", tim->tim_id));
     }
 
+    // set the global variable for interrupt callbacks
+    if (tim->tim_id - 1 < PYB_TIMER_OBJ_ALL_NUM) {
+        MP_STATE_PORT(pyb_timer_obj_all)[tim->tim_id - 1] = tim;
+    }
+
     if (n_args > 1 || n_kw > 0) {
         // start the peripheral
         mp_map_t kw_args;
@@ -655,11 +693,6 @@ STATIC mp_obj_t pyb_timer_make_new(mp_obj_t type_in, mp_uint_t n_args, mp_uint_t
         pyb_timer_init_helper(tim, n_args - 1, args + 1, &kw_args);
     }
 
-    // set the global variable for interrupt callbacks
-    if (tim->tim_id - 1 < PYB_TIMER_OBJ_ALL_NUM) {
-        MP_STATE_PORT(pyb_timer_obj_all)[tim->tim_id - 1] = tim;
-    }
-
     return (mp_obj_t)tim;
 }
 
@@ -686,6 +719,7 @@ STATIC mp_obj_t pyb_timer_deinit(mp_obj_t self_in) {
         prev_chan->next = NULL;
     }
 
+    self->tim.State = HAL_TIM_STATE_RESET;
     self->tim.Instance->CCER = 0x0000; // disable all capture/compare outputs
     self->tim.Instance->CR1 = 0x0000; // disable the timer and reset its state
 
@@ -718,6 +752,9 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(pyb_timer_deinit_obj, pyb_timer_deinit);
 ///     - `Timer.OC_FORCED_ACTIVE` - the pin is forced active (compare match is ignored).
 ///     - `Timer.OC_FORCED_INACTIVE` - the pin is forced inactive (compare match is ignored).
 ///     - `Timer.IC` - configure the timer in Input Capture mode.
+///     - `Timer.ENC_A` --- configure the timer in Encoder mode. The counter only changes when CH1 changes.
+///     - `Timer.ENC_B` --- configure the timer in Encoder mode. The counter only changes when CH2 changes.
+///     - `Timer.ENC_AB` --- configure the timer in Encoder mode. The counter changes when CH1 or CH2 changes.
 ///
 ///   - `callback` - as per TimerChannel.callback()
 ///
@@ -749,6 +786,14 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(pyb_timer_deinit_obj, pyb_timer_deinit);
 ///   Note that capture only works on the primary channel, and not on the
 ///   complimentary channels.
 ///
+/// Notes for Timer.ENC modes:
+///
+///   - Requires 2 pins, so one or both pins will need to be configured to use
+///     the appropriate timer AF using the Pin API.
+///   - Read the encoder value using the timer.counter() method.
+///   - Only works on CH1 and CH2 (and not on CH1N or CH2N)
+///   - The channel number is ignored when setting the encoder mode.
+///
 /// PWM Example:
 ///
 ///     timer = pyb.Timer(2, freq=1000)
@@ -827,7 +872,7 @@ STATIC mp_obj_t pyb_timer_channel(mp_uint_t n_args, const mp_obj_t *pos_args, mp
         const pin_obj_t *pin = pin_obj;
         const pin_af_obj_t *af = pin_find_af(pin, AF_FN_TIM, self->tim_id);
         if (af == NULL) {
-            nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_ValueError, "pin %s doesn't have an af for TIM%d", qstr_str(pin->name), self->tim_id));
+            nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_ValueError, "pin %q doesn't have an af for TIM%d", pin->name, self->tim_id));
         }
         // pin.init(mode=AF_PP, af=idx)
         const mp_obj_t args2[6] = {
@@ -940,6 +985,53 @@ STATIC mp_obj_t pyb_timer_channel(mp_uint_t n_args, const mp_obj_t *pos_args, mp
             break;
         }
 
+        case CHANNEL_MODE_ENC_A:
+        case CHANNEL_MODE_ENC_B:
+        case CHANNEL_MODE_ENC_AB: {
+            TIM_Encoder_InitTypeDef enc_config;
+
+            enc_config.EncoderMode = channel_mode_info[chan->mode].oc_mode;
+            enc_config.IC1Polarity  = args[6].u_int;
+            if (enc_config.IC1Polarity == 0xffffffff) {
+                enc_config.IC1Polarity = TIM_ICPOLARITY_RISING;
+            }
+            enc_config.IC2Polarity  = enc_config.IC1Polarity;
+            enc_config.IC1Selection = TIM_ICSELECTION_DIRECTTI;
+            enc_config.IC2Selection = TIM_ICSELECTION_DIRECTTI;
+            enc_config.IC1Prescaler = TIM_ICPSC_DIV1;
+            enc_config.IC2Prescaler = TIM_ICPSC_DIV1;
+            enc_config.IC1Filter    = 0;
+            enc_config.IC2Filter    = 0;
+
+            if (!IS_TIM_IC_POLARITY(enc_config.IC1Polarity)) {
+                nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_ValueError, "invalid polarity (%d)", enc_config.IC1Polarity));
+            }
+            // Only Timers 1, 2, 3, 4, 5, and 8 support encoder mode
+            if (self->tim.Instance != TIM1
+            &&  self->tim.Instance != TIM2
+            &&  self->tim.Instance != TIM3
+            &&  self->tim.Instance != TIM4
+            &&  self->tim.Instance != TIM5
+            #if defined(TIM8)
+            &&  self->tim.Instance != TIM8
+            #endif
+            ) {
+                nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_ValueError, "encoder not supported on timer %d", self->tim_id));
+            }
+
+            // Disable & clear the timer interrupt so that we don't trigger
+            // an interrupt by initializing the timer.
+            __HAL_TIM_DISABLE_IT(&self->tim, TIM_IT_UPDATE);
+            HAL_TIM_Encoder_Init(&self->tim, &enc_config);
+            __HAL_TIM_SetCounter(&self->tim, 0);
+            if (self->callback != mp_const_none) {
+                __HAL_TIM_CLEAR_FLAG(&self->tim, TIM_IT_UPDATE);
+                __HAL_TIM_ENABLE_IT(&self->tim, TIM_IT_UPDATE);
+            }
+            HAL_TIM_Encoder_Start(&self->tim, TIM_CHANNEL_ALL);
+            break;
+        }
+
         default:
             nlr_raise(mp_obj_new_exception_msg_varg(&mp_type_ValueError, "invalid mode (%d)", chan->mode));
     }
@@ -996,6 +1088,10 @@ STATIC mp_obj_t pyb_timer_freq(mp_uint_t n_args, const mp_obj_t *args) {
         uint32_t prescaler = compute_prescaler_period_from_freq(self, args[1], &period);
         self->tim.Instance->PSC = prescaler;
         __HAL_TIM_SetAutoreload(&self->tim, period);
+        // Reset the counter to zero. Otherwise, if counter >= period it will
+        // continue counting until it wraps (at either 16 or 32 bits depending
+        // on the timer).
+        __HAL_TIM_SetCounter(&self->tim, 0);
         return mp_const_none;
     }
 }
@@ -1026,6 +1122,10 @@ STATIC mp_obj_t pyb_timer_period(mp_uint_t n_args, const mp_obj_t *args) {
     } else {
         // set
         __HAL_TIM_SetAutoreload(&self->tim, mp_obj_get_int(args[1]) & TIMER_CNT_MASK(self));
+        // Reset the counter to zero. Otherwise, if counter >= period it will
+        // continue counting until it wraps (at either 16 or 32 bits depending
+        // on the timer).
+        __HAL_TIM_SetCounter(&self->tim, 0); 
         return mp_const_none;
     }
 }
@@ -1042,10 +1142,13 @@ STATIC mp_obj_t pyb_timer_callback(mp_obj_t self_in, mp_obj_t callback) {
         __HAL_TIM_DISABLE_IT(&self->tim, TIM_IT_UPDATE);
         self->callback = mp_const_none;
     } else if (mp_obj_is_callable(callback)) {
+        __HAL_TIM_DISABLE_IT(&self->tim, TIM_IT_UPDATE);
         self->callback = callback;
+        // start timer, so that it interrupts on overflow, but clear any
+        // pending interrupts which may have been set by initializing it.
+        __HAL_TIM_CLEAR_FLAG(&self->tim, TIM_IT_UPDATE);
+        HAL_TIM_Base_Start_IT(&self->tim); // This will re-enable the IRQ
         HAL_NVIC_EnableIRQ(self->irqn);
-        // start timer, so that it interrupts on overflow
-        HAL_TIM_Base_Start_IT(&self->tim);
     } else {
         nlr_raise(mp_obj_new_exception_msg(&mp_type_ValueError, "callback must be None or a callable object"));
     }
@@ -1076,6 +1179,9 @@ STATIC const mp_map_elem_t pyb_timer_locals_dict_table[] = {
     { MP_OBJ_NEW_QSTR(MP_QSTR_OC_FORCED_ACTIVE),    MP_OBJ_NEW_SMALL_INT(CHANNEL_MODE_OC_FORCED_ACTIVE) },
     { MP_OBJ_NEW_QSTR(MP_QSTR_OC_FORCED_INACTIVE),  MP_OBJ_NEW_SMALL_INT(CHANNEL_MODE_OC_FORCED_INACTIVE) },
     { MP_OBJ_NEW_QSTR(MP_QSTR_IC),                  MP_OBJ_NEW_SMALL_INT(CHANNEL_MODE_IC) },
+    { MP_OBJ_NEW_QSTR(MP_QSTR_ENC_A),               MP_OBJ_NEW_SMALL_INT(CHANNEL_MODE_ENC_A) },
+    { MP_OBJ_NEW_QSTR(MP_QSTR_ENC_B),               MP_OBJ_NEW_SMALL_INT(CHANNEL_MODE_ENC_B) },
+    { MP_OBJ_NEW_QSTR(MP_QSTR_ENC_AB),              MP_OBJ_NEW_SMALL_INT(CHANNEL_MODE_ENC_AB) },
     { MP_OBJ_NEW_QSTR(MP_QSTR_HIGH),                MP_OBJ_NEW_SMALL_INT(TIM_OCPOLARITY_HIGH) },
     { MP_OBJ_NEW_QSTR(MP_QSTR_LOW),                 MP_OBJ_NEW_SMALL_INT(TIM_OCPOLARITY_LOW) },
     { MP_OBJ_NEW_QSTR(MP_QSTR_RISING),              MP_OBJ_NEW_SMALL_INT(TIM_ICPOLARITY_RISING) },
@@ -1098,10 +1204,10 @@ const mp_obj_type_t pyb_timer_type = {
 /// Timer channels are used to generate/capture a signal using a timer.
 ///
 /// TimerChannel objects are created using the Timer.channel() method.
-STATIC void pyb_timer_channel_print(void (*print)(void *env, const char *fmt, ...), void *env, mp_obj_t self_in, mp_print_kind_t kind) {
+STATIC void pyb_timer_channel_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
     pyb_timer_channel_obj_t *self = self_in;
 
-    print(env, "TimerChannel(timer=%u, channel=%u, mode=%s)",
+    mp_printf(print, "TimerChannel(timer=%u, channel=%u, mode=%s)",
           self->timer->tim_id,
           self->channel,
           qstr_str(channel_mode_info[self->mode].name));
@@ -1240,7 +1346,7 @@ STATIC void timer_handle_irq_channel(pyb_timer_obj_t *tim, uint8_t channel, mp_o
                     } else {
                         printf("uncaught exception in Timer(%u) channel %u interrupt handler\n", tim->tim_id, channel);
                     }
-                    mp_obj_print_exception(printf_wrapper, NULL, (mp_obj_t)nlr.ret_val);
+                    mp_obj_print_exception(&mp_plat_print, (mp_obj_t)nlr.ret_val);
                 }
                 gc_unlock();
             }
@@ -1274,8 +1380,9 @@ void timer_irq_handler(uint tim_id) {
 
         // Finally, clear any remaining interrupt sources. Otherwise we'll
         // just get called continuously.
-        uint32_t unhandled = __HAL_TIM_GET_ITSTATUS(&tim->tim, 0xff & ~handled);
+        uint32_t unhandled = tim->tim.Instance->DIER & 0xff & ~handled;
         if (unhandled != 0) {
+            __HAL_TIM_DISABLE_IT(&tim->tim, unhandled);
             __HAL_TIM_CLEAR_IT(&tim->tim, unhandled);
             printf("Unhandled interrupt SR=0x%02lx (now disabled)\n", unhandled);
         }
