@@ -35,12 +35,17 @@
 /* Includes ------------------------------------------------------------------*/
 
 #include <stdbool.h>
+#include <stdint.h>
 
-#include "stm32f4xx_hal.h"
 #include "usbd_cdc_msc_hid.h"
 #include "usbd_cdc_interface.h"
 #include "pendsv.h"
 
+#include "py/obj.h"
+#include "irq.h"
+#include "timer.h"
+#include "usb.h"
+
 // CDC control commands
 #define CDC_SEND_ENCAPSULATED_COMMAND               0x00
 #define CDC_GET_ENCAPSULATED_RESPONSE               0x01
@@ -76,9 +81,6 @@ static uint8_t UserTxNeedEmptyPacket = 0; // used to flush the USB IN endpoint i
 static int user_interrupt_char = -1;
 static void *user_interrupt_data = NULL;
 
-/* USB handler declaration */
-extern USBD_HandleTypeDef hUSBDDevice;
-
 /* Private function prototypes -----------------------------------------------*/
 static int8_t CDC_Itf_Init     (void);
 static int8_t CDC_Itf_DeInit   (void);
@@ -416,6 +418,10 @@ int USBD_CDC_Tx(const uint8_t *buf, uint32_t len, uint32_t timeout) {
                 // timeout
                 return i;
             }
+            if (query_irq() == IRQ_STATE_DISABLED) {
+                // IRQs disabled so buffer will never be drained; return immediately
+                return i;
+            }
             __WFI(); // enter sleep mode, waiting for interrupt
         }
 
@@ -443,6 +449,10 @@ void USBD_CDC_TxAlways(const uint8_t *buf, uint32_t len) {
             // (wraparound of tick is taken care of by 2's complement arithmetic).
             uint32_t start = HAL_GetTick();
             while (((UserTxBufPtrIn + 1) & (APP_TX_DATA_SIZE - 1)) == UserTxBufPtrOut && HAL_GetTick() - start <= 500) {
+                if (query_irq() == IRQ_STATE_DISABLED) {
+                    // IRQs disabled so buffer will never be drained; exit loop
+                    break;
+                }
                 __WFI(); // enter sleep mode, waiting for interrupt
             }
 
@@ -488,6 +498,10 @@ int USBD_CDC_Rx(uint8_t *buf, uint32_t len, uint32_t timeout) {
                 // timeout
                 return i;
             }
+            if (query_irq() == IRQ_STATE_DISABLED) {
+                // IRQs disabled so buffer will never be filled; return immediately
+                return i;
+            }
             __WFI(); // enter sleep mode, waiting for interrupt
         }
 
