@@ -18,11 +18,9 @@
 #include MICROPY_HAL_H
 
 #include "servo.h"
-#include "usb.h"
 #include "led.h"
 #include "uart.h"
 #include "pin.h"
-#include "pybstdio.h"
 
 
 extern uint32_t _heap_start;
@@ -43,8 +41,8 @@ void NORETURN __fatal_error(const char *msg) {
     led_state(2, 1);
     led_state(3, 1);
     led_state(4, 1);
-    stdout_tx_strn("\nFATAL ERROR:\n", 14);
-    stdout_tx_strn(msg, strlen(msg));
+    mp_hal_stdout_tx_strn("\nFATAL ERROR:\n", 14);
+    mp_hal_stdout_tx_strn(msg, strlen(msg));
     for (uint i = 0;;) {
         led_toggle(((i++) & 3) + 1);
         for (volatile uint delay = 0; delay < 10000000; delay++) {
