@@ -1,5 +1,14 @@
 # test user defined iterators
 
+# this class is not iterable
+class NotIterable:
+    pass
+try:
+    for i in NotIterable():
+        pass
+except TypeError:
+    print('TypeError')
+
 class MyStopIteration(StopIteration):
     pass
 
