@@ -49,10 +49,22 @@ try:
 except TypeError:
     print("TypeError")
 
+# enough args, but kw is wrong
+try:
+    t = T(1, baz=3)
+except TypeError:
+    print("TypeError")
+
+# bad argument for member spec
+try:
+    namedtuple('T', 1)
+except TypeError:
+    print("TypeError")
+
 # Try single string
-# Not implemented so far
-#T3 = namedtuple("TupComma", "foo bar")
-#t = T3(1, 2)
+T3 = namedtuple("TupComma", "foo bar")
+t = T3(1, 2)
+print(t.foo, t.bar)
 
 # Try single string with comma field seperator
 # Not implemented so far
