@@ -18,3 +18,13 @@ o.__init__()
 #print(dir(o))
 print(hasattr(o, "attr"))
 print(o.attr)
+
+# should only be able to call __new__ on user types
+try:
+    object.__new__(1)
+except TypeError:
+    print("TypeError")
+try:
+    object.__new__(int)
+except TypeError:
+    print("TypeError")
