@@ -10,7 +10,7 @@ desc = {
 
 data = bytearray(b"01234567")
 
-S = uctypes.struct(desc, uctypes.addressof(data), uctypes.LITTLE_ENDIAN)
+S = uctypes.struct(uctypes.addressof(data), desc, uctypes.LITTLE_ENDIAN)
 
 print(uctypes.sizeof(S.arr))
 assert uctypes.sizeof(S.arr) == 2
