@@ -21,3 +21,5 @@ print(json.dumps((1, (2, 3))))
 print(json.dumps({}))
 print(json.dumps({"a":1}))
 print(json.dumps({"a":(2,[3,None])}))
+print(json.dumps('"quoted"'))
+print(json.dumps('space\n\r\tspace'))
