@@ -1,6 +1,9 @@
 # test struct package with floats
 
-import struct
+try:
+    import ustruct as struct
+except:
+    import struct
 
 i = 1. + 1/2
 # TODO: it looks like '=' format modifier is not yet supported
