@@ -24,3 +24,11 @@ print('%.6f' % strtof('1.23', None))
 
 strtod = libc.func("d", "strtod", "sp")
 print('%.6f' % strtod('1.23', None))
+
+# test passing double and float args
+libm = ffi_open(('libm.so', 'libc.so.0', 'libc.so.6', 'libc.dylib'))
+tgamma = libm.func('d', 'tgamma', 'd')
+tgammaf = libm.func('f', 'tgammaf', 'f')
+for fun in (tgamma, tgammaf):
+    for val in (0.5, 1, 1.0, 1.5, 4, 4.0):
+        print('%.6f' % fun(val))
