@@ -76,6 +76,7 @@ typedef struct _mp_obj_ffifunc_t {
     mp_obj_base_t base;
     void *func;
     char rettype;
+    const char *argtypes;
     ffi_cif cif;
     ffi_type *params[];
 } mp_obj_ffifunc_t;
@@ -166,10 +167,10 @@ STATIC mp_obj_t return_ffi_value(ffi_arg val, char type)
 
 // FFI module
 
-STATIC void ffimod_print(void (*print)(void *env, const char *fmt, ...), void *env, mp_obj_t self_in, mp_print_kind_t kind) {
+STATIC void ffimod_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
     (void)kind;
     mp_obj_ffimod_t *self = self_in;
-    print(env, "<ffimod %p>", self->handle);
+    mp_printf(print, "<ffimod %p>", self->handle);
 }
 
 STATIC mp_obj_t ffimod_close(mp_obj_t self_in) {
@@ -181,6 +182,7 @@ STATIC MP_DEFINE_CONST_FUN_OBJ_1(ffimod_close_obj, ffimod_close);
 
 STATIC mp_obj_t make_func(mp_obj_t rettype_in, void *func, mp_obj_t argtypes_in) {
     const char *rettype = mp_obj_str_get_str(rettype_in);
+    const char *argtypes = mp_obj_str_get_str(argtypes_in);
 
     mp_int_t nparams = MP_OBJ_SMALL_INT_VALUE(mp_obj_len_maybe(argtypes_in));
     mp_obj_ffifunc_t *o = m_new_obj_var(mp_obj_ffifunc_t, ffi_type*, nparams);
@@ -188,6 +190,7 @@ STATIC mp_obj_t make_func(mp_obj_t rettype_in, void *func, mp_obj_t argtypes_in)
 
     o->func = func;
     o->rettype = *rettype;
+    o->argtypes = argtypes;
 
     mp_obj_t iterable = mp_getiter(argtypes_in);
     mp_obj_t item;
@@ -335,10 +338,10 @@ STATIC const mp_obj_type_t ffimod_type = {
 
 // FFI function
 
-STATIC void ffifunc_print(void (*print)(void *env, const char *fmt, ...), void *env, mp_obj_t self_in, mp_print_kind_t kind) {
+STATIC void ffifunc_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
     (void)kind;
     mp_obj_ffifunc_t *self = self_in;
-    print(env, "<ffifunc %p>", self->func);
+    mp_printf(print, "<ffifunc %p>", self->func);
 }
 
 STATIC mp_obj_t ffifunc_call(mp_obj_t self_in, mp_uint_t n_args, mp_uint_t n_kw, const mp_obj_t *args) {
@@ -348,9 +351,20 @@ STATIC mp_obj_t ffifunc_call(mp_obj_t self_in, mp_uint_t n_args, mp_uint_t n_kw,
 
     ffi_arg values[n_args];
     void *valueptrs[n_args];
-    for (uint i = 0; i < n_args; i++) {
+    const char *argtype = self->argtypes;
+    for (uint i = 0; i < n_args; i++, argtype++) {
         mp_obj_t a = args[i];
-        if (a == mp_const_none) {
+        if (*argtype == 'O') {
+            values[i] = (ffi_arg)a;
+        #if MICROPY_PY_BUILTINS_FLOAT
+        } else if (*argtype == 'f') {
+            float *p = (float*)&values[i];
+            *p = mp_obj_get_float(a);
+        } else if (*argtype == 'd') {
+            double *p = (double*)&values[i];
+            *p = mp_obj_get_float(a);
+        #endif
+        } else if (a == mp_const_none) {
             values[i] = 0;
         } else if (MP_OBJ_IS_INT(a)) {
             values[i] = mp_obj_int_get_truncated(a);
@@ -361,7 +375,7 @@ STATIC mp_obj_t ffifunc_call(mp_obj_t self_in, mp_uint_t n_args, mp_uint_t n_kw,
             mp_obj_base_t *o = (mp_obj_base_t*)a;
             mp_buffer_info_t bufinfo;
             int ret = o->type->buffer_p.get_buffer(o, &bufinfo, MP_BUFFER_READ); // TODO: MP_BUFFER_READ?
-            if (ret != 0 || bufinfo.buf == NULL) {
+            if (ret != 0) {
                 goto error;
             }
             values[i] = (ffi_arg)bufinfo.buf;
@@ -404,10 +418,10 @@ STATIC const mp_obj_type_t ffifunc_type = {
 
 // FFI callback for Python function
 
-STATIC void fficallback_print(void (*print)(void *env, const char *fmt, ...), void *env, mp_obj_t self_in, mp_print_kind_t kind) {
+STATIC void fficallback_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
     (void)kind;
     mp_obj_fficallback_t *self = self_in;
-    print(env, "<fficallback %p>", self->func);
+    mp_printf(print, "<fficallback %p>", self->func);
 }
 
 STATIC const mp_obj_type_t fficallback_type = {
@@ -418,11 +432,11 @@ STATIC const mp_obj_type_t fficallback_type = {
 
 // FFI variable
 
-STATIC void ffivar_print(void (*print)(void *env, const char *fmt, ...), void *env, mp_obj_t self_in, mp_print_kind_t kind) {
+STATIC void ffivar_print(const mp_print_t *print, mp_obj_t self_in, mp_print_kind_t kind) {
     (void)kind;
     mp_obj_ffivar_t *self = self_in;
     // Variable value printed as cast to int
-    print(env, "<ffivar @%p: 0x%x>", self->var, *(int*)self->var);
+    mp_printf(print, "<ffivar @%p: 0x%x>", self->var, *(int*)self->var);
 }
 
 STATIC mp_obj_t ffivar_get(mp_obj_t self_in) {
